<?php

require_once 'controllers/default/default.php'; // Zakladny contoller

class admin_controller extends default_controller {
  
  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    parent::__construct($fw);
    
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'admin_interface')) {
      $this->_fw->sessions->setFlashMessage('login_message', 'Nemáte prístup do administrácie.', 'error');
      $this->_fw->redirectTo('login', 'index');
    }
    
    $this->_fw->resources->getGroup('editor')->allowResource('products')->allowResource('orders');
    $this->_fw->resources->getGroup('admin')->allowResource('users');
    
    $admin_menu = array();
    $admin_menu[] = array('Index', $this->_fw->makeLinkUrl('admin', 'index'), 'index');
    if ($this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'products')) {
      $admin_menu[] = array('Produkty', $this->_fw->makeLinkUrl('admin', 'products'), 'products');
    }
    if ($this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'orders')) {
      $admin_menu[] = array('Objednávky', $this->_fw->makeLinkUrl('admin', 'orders'), 'orders');
    }
    if ($this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'users')) {
      $admin_menu[] = array('Použivatelia', $this->_fw->makeLinkUrl('admin', 'users'), 'users');
    }
    
    $this->_fw->templateAssign('admin_menu', $admin_menu);
    $this->_fw->templateAssign('active_menu_item', 'admin');
    $this->_fw->templateAssign('message', $this->_fw->sessions->getFlashMessage('admin_message'));
  }
 
  /*
   * Hlavna akcia
   * navrat: nic   
   */   
  public function index_action() {
    $this->_fw->templateAssign('admin_menu_item', 'index');
    
    $this->_fw->templateDisplay('admin/index.html');
  } 
  
  /*
   * Produkty
   * navrat: nic   
   */   
  public function products_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'products')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu produktov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $this->_fw->loadModel('products');
    $this->_fw->loadModel('pagination');
    
    $pid = $this->_fw->pagination->createNewPagination('uwa_produkty', 10);
    $this->_fw->pagination->setUniqueVarname($pid, 'admin_products_page');
    
    $this->_fw->pagination->setTemplateMarkers($pid, $this->_fw->makeLinkUrl('admin', 'products'));
    
    $this->_fw->templateAssign('admin_menu_item', 'products');
    
    $this->_fw->templateAssign('new_product_action', $this->_fw->makeLinkUrl('admin', 'new_product'));
    $this->_fw->templateAssign('products', $this->_fw->products->getAllProducts($pid));
    
    $this->_fw->templateAssign('link_edit', $this->_fw->makeLinkUrl('admin', 'edit_product', array('product_id' => '_product_id_')));
    $this->_fw->templateAssign('link_delete', $this->_fw->makeLinkUrl('admin', 'delete_product', array('product_id' => '_product_id_')));
    
    $this->_fw->templateDisplay('admin/products.html');
  } 
  
  /*
   * Vytvorenie noveho produktu
   * navrat: nic   
   */   
  public function new_product_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'products')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu produktov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    if (isset($_POST['product']) && isset($_POST['product_submit'])) {
      $this->_fw->loadModel('products');
      $result = $this->_fw->products->verifyProductForm($_POST['product']);
      if ($result === true) {
        if ($this->_fw->products->createNewProduct($_POST['product'])) {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Produkt úspešne vytvorený.', 'success');
          $this->_fw->redirectTo('admin', 'products');
        } else {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Nepodarilo sa vytvoriť produkt. Databazova chyba.', 'error');
          $this->_fw->callControllerAction('admin', 'products');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('admin_message', $result, 'error');
        $this->_fw->callControllerAction('admin', 'products');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Neboli odoslané všetky náležité informácie na vytvorenie produktu.', 'error');
      $this->_fw->callControllerAction('admin', 'products');
    }
  }
  
  /*
   * Uprava produktu
   * navrat: nic   
   */   
  public function edit_product_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'products')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu produktov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (!isset($data['product_id'])) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebol zadaný produkt.', 'error');
      $this->_fw->redirectTo('admin', 'products');
    }
    
    $this->_fw->loadModel('products');
    
    if (isset($_POST['product'])) {
      $this->_fw->templateAssign('product', $_POST['product']);
    } else {
      $row = $this->_fw->products->getSingleProduct($data['product_id']);
      if (count($row) == 0) {
        $this->_fw->sessions->setFlashMessage('admin_message', 'Zadaný produkt neexistuje.', 'error');
        $this->_fw->redirectTo('admin', 'products');
      }
      $this->_fw->templateAssign('product', $row);
    }
    
    $this->_fw->templateAssign('admin_menu_item', 'products');
    $this->_fw->templateAssign('edit_product_action', $this->_fw->makeLinkUrl('admin', 'do_edit_product', array('product_id' => $data['product_id'])));
    
    $this->_fw->templateDisplay('admin/edit_product.html');
  }
  
  /*
   * Vykona upravu produktu
   * navrat: nic   
   */   
  public function do_edit_product_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'products')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu produktov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (!isset($data['product_id'])) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebol zadaný produkt.', 'error');
      $this->_fw->redirectTo('admin', 'products');
    }
    
    $this->_fw->loadModel('products');
    
    if (isset($_POST['product']) && isset($_POST['product_submit'])) {
      $result = $this->_fw->products->verifyProductForm($_POST['product']);
      if ($result === true) {
        if ($this->_fw->products->updateProduct($data['product_id'], $_POST['product'])) {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Úprava produktu úspešná.', 'success');
          $this->_fw->redirectTo('admin', 'products');
        } else {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Neboli vykonané žiadne zmeny.', 'error');
          $this->_fw->callControllerAction('admin', 'edit_product');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('admin_message', $result, 'error');
        $this->_fw->callControllerAction('admin', 'edit_product');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Neboli odoslané žiadne dáta na zmenu údajov produktu.', 'error');
      $this->_fw->callControllerAction('admin', 'edit_product');
    }
  }
  
  /*
   * Vymaze produkt
   * navrat: nic   
   */   
  public function delete_product_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'products')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu produktov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (!isset($data['product_id'])) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebol zadaný produkt.', 'error');
      $this->_fw->redirectTo('admin', 'products');
    }
    
    $this->_fw->loadModel('products');
    
    if ($this->_fw->products->deleteProduct($data['product_id'])) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Produkt bol úspešne vymazaný.', 'success');
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Produkt nebol vymazaný.', 'error');
    }
    $this->_fw->redirectTo('admin', 'products');
  }
  
  /*
   * Objednavky
   * navrat: nic   
   */   
  public function orders_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'orders')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu objednávok.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
  
    $this->_fw->templateAssign('admin_menu_item', 'orders');
    
    $this->_fw->loadModel('orders');
    $this->_fw->loadModel('pagination');
    
    $pid0 = $this->_fw->pagination->createNewPagination('uwa_objednavky', 10, 'filterByStatus0');
    $pid1 = $this->_fw->pagination->createNewPagination('uwa_objednavky', 10, 'filterByStatus1');
    $pid2 = $this->_fw->pagination->createNewPagination('uwa_objednavky', 10, 'filterByStatus2');
    
    $link = $this->_fw->makeLinkUrl('admin', 'orders');
    $this->_fw->pagination->setTemplateMarkers($pid0, $link, 'status0');
    $this->_fw->pagination->setTemplateMarkers($pid1, $link, 'status1');
    $this->_fw->pagination->setTemplateMarkers($pid2, $link, 'status2');
    
    $this->_fw->templateAssign('orders_0', $this->_fw->orders->getAllOrdersByStatus(0, $pid0));
    $this->_fw->templateAssign('orders_1', $this->_fw->orders->getAllOrdersByStatus(1, $pid1));
    $this->_fw->templateAssign('orders_2', $this->_fw->orders->getAllOrdersByStatus(2, $pid2));
    
    $this->_fw->templateAssign('link_edit', $this->_fw->makeLinkUrl('admin', 'edit_order', array('order_id' => '_order_id_')));
    $this->_fw->templateAssign('link_delete', $this->_fw->makeLinkUrl('admin', 'delete_order', array('order_id' => '_order_id_')));
    
    $this->_fw->templateDisplay('admin/orders.html');
  }
  
  /*
   * Upravenie objednávky
   * navrat: nic   
   */   
  public function edit_order_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'orders')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu objednávok.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (!isset($data['order_id'])) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebola zadaná objednávka.', 'error');
      $this->_fw->redirectTo('admin', 'orders');
    }
    
    $this->_fw->loadModel('orders');
    
    if (isset($_POST['order'])) {
      $this->_fw->templateAssign('order', $_POST['order']);
    } else {
      $row = $this->_fw->orders->getSingleOrder($data['order_id']);
      if (count($row) == 0) {
        $this->_fw->sessions->setFlashMessage('admin_message', 'Zadaná objednávka neexistuje.', 'error');
        $this->_fw->redirectTo('admin', 'orders');
      }
      $this->_fw->templateAssign('order', $row);
    }
    
    $payment_options = array(
      array('caption' => 'Na dobierku.', 'value' => '1'),
      array('caption' => 'Prevodom na účet.', 'value' => '2'),
      array('caption' => 'Osobný odber.', 'value' => '3'),
    );
    
    $status_options = array(
      array('value' => 0, 'caption' => 'Prijatá na spracovanie'), 
      array('value' => 1, 'caption' => 'Spracovávaná'), 
      array('value' => 2, 'caption' => 'Dokončená'),
    );
    
    $this->_fw->templateAssign('payment_options', $payment_options);
    $this->_fw->templateAssign('status_options', $status_options);
    
    $this->_fw->loadModel('users');
    
    $this->_fw->templateAssign('users_options', $this->_fw->users->getAllUsers());
    
    $this->_fw->templateAssign('edit_order_action', $this->_fw->makeLinkUrl('admin', 'do_edit_order', array('order_id' => $data['order_id'])));
    
    $this->_fw->templateDisplay('admin/edit_order.html');
  }
  
  /*
   * Vykana upravu objednavky
   * navrat: nic   
   */   
  public function do_edit_order_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'orders')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu objednávok.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (!isset($data['order_id'])) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebola zadaná objednávka.', 'error');
      $this->_fw->redirectTo('admin', 'orders');
    }
    
    $this->_fw->loadModel('orders');
    
    if (isset($_POST['order']) && isset($_POST['order_submit'])) {
      $result = $this->_fw->orders->adminVerifyOrderData($_POST['order']);
      if ($result === true) {
        if ($this->_fw->orders->adminUpdateOrder($data['order_id'], $_POST['order'])) {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Úprava objednávky úspešná.', 'success');
          $this->_fw->redirectTo('admin', 'orders');
        } else {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Neboli vykonané žiadne zmeny.', 'error');
          $this->_fw->callControllerAction('admin', 'edit_order');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('admin_message', $result, 'error');
        $this->_fw->callControllerAction('admin', 'edit_order');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Neboli odoslané žiadne dáta na zmenu údajov objednávky.', 'error');
      $this->_fw->callControllerAction('admin', 'edit_order');
    }
  }
  
  /*
   * Vymazanie objednavky
   * navrat: nic   
   */   
  public function delete_order_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'orders')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu objednávok.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (!isset($data['order_id'])) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebola zadaná objednávka.', 'error');
      $this->_fw->redirectTo('admin', 'orders');
    }
    
    $this->_fw->loadModel('orders');
    
    if ($this->_fw->orders->deleteOrder($data, $order)) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Objednávka bola úspešne vymazaná.', 'success');
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Objednávka nebola vymazaná.', 'error');
    }
    $this->_fw->redirectTo('admin', 'orders');
  }
  
  /*
   * Pouzivatelia
   * navrat: nic   
   */   
  public function users_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'users')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu použivateľov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
  
    $this->_fw->templateAssign('admin_menu_item', 'users');
    
    $this->_fw->loadModel('users');
    $this->_fw->loadModel('pagination');
    
    $pid = $this->_fw->pagination->createNewPagination('uwa_pouzivatelia', 10);
    $this->_fw->pagination->setUniqueVarname($pid, 'admin_users_page');
    
    $this->_fw->pagination->setTemplateMarkers($pid, $this->_fw->makeLinkUrl('admin', 'users'));
    
    $groups_options = array(array('tag' => 'user', 'name' => 'Použivateľ'), array('tag' => 'editor', 'name' => 'Editor'), array('tag' => 'admin', 'name' => 'Administrátor'));
    
    $this->_fw->templateAssign('groups_options', $groups_options);
    $this->_fw->templateAssign('users', $this->_fw->users->getAllUsers());
    
    $this->_fw->templateAssign('new_user_action', $this->_fw->makeLinkUrl('admin', 'create_user'));
    $this->_fw->templateAssign('link_edit', $this->_fw->makeLinkUrl('admin', 'edit_user', array('user_id' => '_user_id_')));
    $this->_fw->templateAssign('link_delete', $this->_fw->makeLinkUrl('admin', 'delete_user', array('user_id' => '_user_id_')));
    
    $this->_fw->templateDisplay('admin/users.html');
  }
  
  /*
   * Vytvorenie pouzivatela
   * navrat: nic   
   */   
  public function create_user_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'users')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu použivateľov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $this->_fw->loadModel('users');
    
    if (isset($_POST['user']) && isset($_POST['user_submit'])) {
      $result = $this->_fw->users->validateAdminForm(0, $_POST['user'], true);
      if ($result === true) {
        if ($this->_fw->users->adminCreateUser($_POST['user'])) {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Nový použivateľ bol úspešne vytvorený.', 'success');
          $this->_fw->redirectTo('admin', 'users');
        } else {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Nepodarilo sa vytvoriť nového použivateľa.', 'error');
          $this->_fw->callControllerAction('admin', 'users');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('admin_message', $result, 'error');
        $this->_fw->callControllerAction('admin', 'users');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Neboli odoslané všetky náležité údaje na vytvorenie použivateľa.', 'error');
      $this->_fw->callControllerAction('admin', 'users');
    }
  }
  
  /*
   * Uprava pouzivatela
   * navrat: nic   
   */   
  public function edit_user_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'users')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu použivateľov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (isset($data['user_id']) && intval($data['user_id']) > 0) {
      $this->_fw->loadModel('users');
      
      if (isset($_POST['user'])) {
        $this->_fw->templateAssign('user', $_POST['user']);
      } else {
        $this->_fw->templateAssign('user', $this->_fw->users->getSingleUser($data['user_id']));
      }
      
      $userdata = $this->_fw->users->getUserData();
      
      if ($userdata['id'] == $data['user_id']) { $this->_fw->templateAssign('warning', true); }
      
      $groups_options = array(array('tag' => 'user', 'name' => 'Použivateľ'), array('tag' => 'editor', 'name' => 'Editor'), array('tag' => 'admin', 'name' => 'Administrátor'));
    
      $this->_fw->templateAssign('groups_options', $groups_options);
      
      $this->_fw->templateAssign('edit_user_action', $this->_fw->makeLinkUrl('admin', 'do_edit_user', $data));
      
      $this->_fw->templateDisplay('admin/edit_user.html');
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebolo zadané ID použivatela na úpravu.', 'error');
      $this->_fw->redirectTo('admin', 'users');
    }
  }
  
  public function do_edit_user_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'users')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu použivateľov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (isset($data['user_id']) && intval($data['user_id']) > 0) {
      $this->_fw->loadModel('users');
      
      $result = $this->_fw->users->validateAdminForm($data['user_id'], $_POST['user'], false);
      if ($result === true) {
        if ($this->_fw->users->editUser($data['user_id'], $_POST['user'])) {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Použivateľ úspešne upravený.', 'success');
          $this->_fw->redirectTo('admin', 'users');
        } else {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Nebola vykonaná žiadna zmena.', 'error');
          $this->_fw->redirectTo('admin', 'users');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('admin_message', $result, 'error');
        $this->_fw->callControllerAction('admin', 'edit_user');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebolo zadané ID použivatela na úpravu.', 'error');
      $this->_fw->redirectTo('admin', 'users');
    }
  }
  
  /*
   * Vymaze pouzivatela
   * navrat: nic   
   */   
  public function delete_user_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'users')) {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nemáte oprávnenie na správu použivateľov.', 'error');
      $this->_fw->redirectTo('admin', 'index');
    }
    
    $data = $this->_fw->getUrlParams();
    
    if (isset($data['user_id']) && intval($data['user_id']) > 0) {
      $this->_fw->loadModel('users');
      $userdata = $this->_fw->users->getUserData();
      
      if ($userdata['id'] != $data['user_id']) {
        if ($this->_fw->users->deleteUser($data['user_id'])) {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Použivateľ bol úspešne vymazaný.', 'success');
          $this->_fw->redirectTo('admin', 'users');
        } else {
          $this->_fw->sessions->setFlashMessage('admin_message', 'Pri vymazávaní použivateľa sa vyskytla chyba.', 'error');
          $this->_fw->redirectTo('admin', 'users');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('admin_message', 'Nie je možné vymazať tento použivateľský účet.', 'error');
        $this->_fw->redirectTo('admin', 'users');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('admin_message', 'Nebolo zadané ID použivatela na zmazanie.', 'error');
      $this->_fw->redirectTo('admin', 'users');
    }
  }
}