<?php

require_once 'controllers/default/default.php'; // Zakladny contoller

class cart_controller extends default_controller {
  
  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    parent::__construct($fw);
    
    $this->_fw->loadModel('products');
    $this->_fw->loadModel('cart');
    $this->_fw->loadModel('users');
    
    $this->_fw->templateAssign('page_title', 'Košík');
    $this->_fw->templateAssign('active_menu_item', 'kosik');
    
    $this->_fw->resources->getGroup('user')->allowResource('checkout');
  }
  
  /*
   * Akcia "index"
   * navrat: nic   
   */   
  public function index_action() {
    $this->_fw->templateAssign('add_to_cart_message', $this->_fw->sessions->getFlashMessage('add_to_cart'));
    
    $this->_fw->templateAssign('cart', $this->_fw->cart->getCartProducts());
    
    $this->_fw->templateAssign('link_to_products', $this->_fw->makeLinkUrl('products', 'detail', array('product' => '_product_id_')));
    $this->_fw->templateAssign('link_to_delete', $this->_fw->makeLinkUrl('cart', 'delete_from_cart', array('product' => '_product_id_')));
    $this->_fw->templateAssign('link_to_update_count', $this->_fw->makeLinkUrl('cart', 'update_count', array('product' => '_product_id_')));
    $this->_fw->templateAssign('link_to_checkout', $this->_fw->makeLinkUrl('cart', 'checkout'));
    
    $this->_fw->templateAssign('allow_checkout', $this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'checkout'));
    
    $this->_fw->templateDisplay('cart/index.html');
  }
  
  /*
   * Akcia "add_to_cart"
   * navrat: nic   
   */
  public function add_to_cart_action() {
    if (isset($_POST['product_submit']) && isset($_POST['product_id']) && is_numeric($_POST['product_id'])
        && isset($_POST['product_count']) && is_numeric($_POST['product_count'])) {
      if ($this->_fw->products->productExist($_POST['product_id'])) {
        if ($this->_fw->cart->addToCart($_POST['product_id'], $_POST['product_count'])) {
          $this->_fw->sessions->setFlashMessage('add_to_cart', 'Produkt bol vložený do košíka.', 'success');
        } else {
          $this->_fw->sessions->setFlashMessage('add_to_cart', 'Produkt nie je možné vložiť do košíka pri nastavení počtu ' . $_POST['product_count'] . ' kusov.', 'error');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('add_to_cart', 'Produkt, ktorý sa snažíte vložiť do košíka, neexistuje.', 'error');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('add_to_cart', 'Chybný vstup. Nie je možné vložiť produkt do košíka!', 'error');
    }
    $this->_fw->redirectTo('cart', 'index');
  }
  
  /*
   * Akcia "delete_from_cart"
   * navrat: nic   
   */
  public function delete_from_cart_action() {
    $data = $this->_fw->getUrlParams();
    
    if (isset($data['product']) && is_numeric($data['product'])) {
      if ($this->_fw->cart->deleteFromCart($data['product'])) {
        $this->_fw->sessions->setFlashMessage('add_to_cart', 'Produkt bol vymazaný z košíka.', 'success');
      } else {
        $this->_fw->sessions->setFlashMessage('add_to_cart', 'Produkt sa v košíku už nenachádza, nie je ho možné opäť vyhodiť.', 'error');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('add_to_cart', 'Nebol zadaný produkt, ktorý sa má vyhodiť z košíka.', 'error');
    }
    
    $this->_fw->redirectTo('cart', 'index');
  }
  
  /*
   * Akcia "update_count"
   * navrat: nic   
   */
  public function update_count_action() {
    $data = $this->_fw->getUrlParams();
    
    if (isset($data['product']) && is_numeric($data['product'])) {
      $cart_product = $this->_fw->cart->getSingleCartProduct($data['product']);
      if (count($cart_product)) {
        $this->_fw->templateAssign('product', $cart_product);
        $this->_fw->templateAssign('update_count_action', $this->_fw->makeLinkUrl('cart', 'do_update_count'));
      } else {
        $this->_fw->sessions->setFlashMessage('add_to_cart', 'Požadovaný produkt sa v košíku nenachádza.', 'error');
        $this->_fw->redirectTo('cart', 'index');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('add_to_cart', 'Nebol zadaný žiadny produkt.', 'error');
      $this->_fw->redirectTo('cart', 'index');
    }
    
    $this->_fw->templateDisplay('cart/update_count.html');
  }
  
  /*
   * Akcia "do_update"
   * navrat: nic   
   */
  public function do_update_count_action() {
    if (isset($_POST['product_submit']) && isset($_POST['product_id']) && is_numeric($_POST['product_id'])
        && isset($_POST['product_count']) && is_numeric($_POST['product_count'])) {
      if ($this->_fw->cart->updateProductCount($_POST['product_id'], $_POST['product_count'])) {
        $this->_fw->sessions->setFlashMessage('add_to_cart', 'Počet objednaných produktov zmenený.', 'success');
      } else {
        $this->_fw->sessions->setFlashMessage('add_to_cart', 'Nie je možné upraviť počet produktov, ktoré nie sú v košíku.', 'error');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('add_to_cart', 'Chybný vstup. Nie je možné upraviť počet produktov v košíku!', 'error');
    }
    
    $this->_fw->redirectTo('cart', 'index');
  }
  
  /*
   * Akcia "checkout"
   * navrat: nic   
   */
  public function checkout_action() {
    $this->_fw->templateAssign('page_title', 'Pokladňa');
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'checkout')) {
      $this->_fw->sessions->setFlashMessage('add_to_cart', 'Musíte sa najprv ' . $this->_fw->makeLinkElement('prihlásiť', 'login', 'index', array(), array('class'=>'highlight')) . ' než budete môcť pokračovať v nákupe.', 'error');
      $this->_fw->redirectTo('cart', 'index');
    }
    $products = $this->_fw->cart->getCartProducts();
    
    if ($this->_fw->cart->getIsCartEmpty()) {
      $this->_fw->sessions->setFlashMessage('add_to_cart', 'Košík je prázdny. Nie je možné pokračovať v objednávke.', 'error');
      $this->_fw->redirectTo('cart', 'index');
    }
    
    $this->_fw->templateAssign('message', $this->_fw->sessions->getFlashMessage('checkout'));
    $this->_fw->templateAssign('cart', $products);
    $this->_fw->templateAssign('checkout_action', $this->_fw->makeLinkUrl('cart', 'do_checkout'));
    
    $accept_terms_array = array(
      array('caption' => 'Prijímam obchodné podmienky.', 'value' => '1'),
    );
    $this->_fw->templateAssign('accept_terms_array', $accept_terms_array);
    
    $payment_options = array(
      array('caption' => 'Na dobierku.', 'value' => '1'),
      array('caption' => 'Prevodom na účet.', 'value' => '2'),
      array('caption' => 'Osobný odber.', 'value' => '3'),
    );
    $this->_fw->templateAssign('payment_options', $payment_options);
    
    $this->_fw->templateDisplay('cart/checkout.html');
  }
  
  /*
   * Akcia "do_checkout"
   * navrat: nic   
   */
  public function do_checkout_action() {
    if (!$this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'checkout')) {
      $this->_fw->sessions->setFlashMessage('add_to_cart', 'Musíte sa najprv ' . $this->_fw->makeLinkElement('prihlásiť', 'login', 'index', array(), array('class'=>'highlight')) . ' než budete môcť pokračovať v nákupe.', 'error');
      $this->_fw->redirectTo('cart', 'index');
    }
    
    if (isset($_POST['submit_checkout']) && isset($_POST['meno']) && isset($_POST['email']) && isset($_POST['adresa'])) {
      $meno = trim($_POST['meno']);
      $email = trim($_POST['email']);
      $adresa = trim($_POST['adresa']);
      $platba = isset($_POST['platba']) && is_numeric($_POST['platba']) ? trim($_POST['platba']) : 0;
      $poznamka = isset($_POST['poznamka']) ? trim($_POST['poznamka']) : '';
      $result = $this->_fw->cart->verifyOrderData($meno, $email, $adresa, $platba);
      if (!isset($_POST['podmienky']) || !in_array('1', $_POST['podmienky'])) {
        $msg = 'Musíte súhlasiť s obchodnými podmienkami!';
        $result = $result === true ? $msg : $result . '<br />' . $msg;
      }
      if ($result === true) {
        if ($this->_fw->cart->storeNewOrder($meno, $email, $adresa, $platba, $poznamka)) {
          $this->_fw->sessions->setFlashMessage('login_message', 'Objednávka bola prijatá.', 'success');
          $this->_fw->redirectTo('login', 'logedin');
        } else {
          $this->_fw->sessions->setFlashMessage('checkout', 'Objednávka nebola prijatá. Vyskytla sa chyba v databázovom spojení, opakujte pokus neskôr.', 'error');
          $this->_fw->redirectTo('cart', 'checkout');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('checkout', $result, 'error');
        $this->_fw->callControllerAction('cart', 'checkout');
        return;
      }
    } else {
      $this->_fw->sessions->setFlashMessage('checkout', 'Nie je mozné pokračovať v objednávke. Systém neobdržal požadované informácie.', 'error');
      $this->_fw->callControllerAction('cart', 'checkout');
      return;
    }
  }
  
}