<?php

require_once 'controllers/default/default.php'; // Zakladny contoller

class login_controller extends default_controller {
  
  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    parent::__construct($fw);
    
    $this->_fw->loadModel('users');
    $this->_fw->loadModel('sessions');
    
    $this->_fw->templateAssign('page_title', 'Prihlásenie');
    $this->_fw->templateAssign('active_menu_item', 'prihlasenie');
    
    $this->_fw->resources->getGroup('user')
              ->allowResource('edit_order')
              ->allowResource('delete_order')
              ->allowResource('change_userdata');
  }
  
  /*
   * Akcia "index"
   * navrat: nic   
   */ 
  public function index_action() {
    if ($this->_fw->users->isUserLogedIn()) { $this->_fw->redirectTo('login', 'logedin'); }
    
    $this->_fw->templateAssign('login_action', $this->_fw->makeLinkUrl('login', 'do_login'));
    $this->_fw->templateAssign('registration_link', $this->_fw->makeLinkUrl('login', 'registration'));
    $this->_fw->templateAssign('message', $this->_fw->sessions->getFlashMessage('login_message'));
    
    $this->_fw->templateDisplay('login/index.html');
  }
  
  /*
   * Akcia "logedin"
   * navrat: nic   
   */ 
  public function logedin_action() {
    if (!$this->_fw->users->isUserLogedIn()) { $this->_fw->redirectTo('login', 'index'); }
    
    $this->_fw->loadModel('orders');
    
    $this->_fw->templateAssign('page_title', 'Použivateľ');
    
    $userdata = $this->_fw->users->getUserData();
    
    if (isset($_POST['change_filter']) && isset($_POST['filter']) && is_numeric($_POST['filter']) && $_POST['filter'] >= -1 && $_POST['filter'] <= 2) {
      $filter = $_POST['filter'];
      $this->_fw->sessions->storeUserDataGroup('frontend_orders_filter', $filter);
    } else {
      $filter = $this->_fw->sessions->restoreUserDataGroup('frontend_orders_filter');
      if ($filter === '') {
        $filter = -1;
        $this->_fw->sessions->storeUserDataGroup('frontend_orders_filter', $filter);
      }
    }
    
    $this->_fw->loadModel('pagination');
    $pid = $this->_fw->pagination->createNewPagination('uwa_objednavky', 10);
    $this->_fw->pagination->setUniqueVarname($pid, 'login_orders_page');
    $this->_fw->pagination->forceSetNumRows($pid, $this->_fw->orders->getOrdersNumByUserId($userdata['id'], $filter));
    
    $this->_fw->pagination->setTemplateMarkers($pid, $this->_fw->makeLinkUrl('login', 'logedin'));
    
    $this->_fw->templateAssign('userdata', $userdata);
    $this->_fw->templateAssign('filter', $filter);
    $this->_fw->templateAssign('orders', $this->_fw->users->getUserOrders($filter, $pid));
    $this->_fw->templateAssign('message', $this->_fw->sessions->getFlashMessage('login_message'));
    $this->_fw->templateAssign('logout_action', $this->_fw->makeLinkUrl('login', 'logout'));
    $this->_fw->templateAssign('payments', array(1 => 'Na dobierku.', 'Prevod na účet.', 'Osobný odber.'));
    $this->_fw->templateAssign('status', array(array('id' => 0, 'text' => 'Prijatá na spracovanie'), array('id' => 1, 'text' => 'Spracovávaná'), array('id' => 2, 'text' => 'Dokončená')));
    $this->_fw->templateAssign('link_to_edit', $this->_fw->makeLinkUrl('login', 'edit_order', array('order_id' => '___order_id___')));
    $this->_fw->templateAssign('link_to_delete', $this->_fw->makeLinkUrl('login', 'delete_order', array('order_id' => '___order_id___')));
    $this->_fw->templateAssign('dateformat', 'j. M Y H:i:s');
    
    $this->_fw->templateAssign('allow_edit', $this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'edit_order'));
    $this->_fw->templateAssign('allow_delete', $this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'delete_order'));
    $this->_fw->templateAssign('allow_datachange', $this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'change_userdata'));
    
    $this->_fw->templateAssign('filter_action', $this->_fw->makeLinkUrl('login', 'logedin'));
    $this->_fw->templateAssign('change_action', $this->_fw->makeLinkUrl('login', 'changedata'));
    
    $this->_fw->templateDisplay('login/logedin.html');
  }
  
  /*
   * Akcia "do_login"
   * navrat: nic   
   */ 
  public function do_login_action() {
    if ($this->_fw->users->isUserLogedIn()) {
      $this->_fw->sessions->setFlashMessage('login_message', 'Prihlasovanie zlyhalo, už ste prihlásený.', 'error');
      $this->_fw->redirectTo('login', 'logedin');
    }
    
    if (isset($_POST['submit_login']) && isset($_POST['login']) && isset($_POST['password'])) {
      if ($this->_fw->users->logInUser($_POST['login'], ($_POST['password']))) {
        $this->_fw->sessions->setFlashMessage('login_message', 'Prihlásenie bolo úspešné. Vitajte!', 'success');
        $this->_fw->redirectTo('login', 'logedin');
      } else {                            
        $this->_fw->sessions->setFlashMessage('login_message', 'Neznáme meno použivatela alebo heslo.', 'error');
        $this->_fw->callControllerAction('login', 'index');
        return;
      }
    } else {
      $this->_fw->sessions->setFlashMessage('login_message', 'Nie je možné pokračovať v prihlasovaní. Zadajte svoje prihlasovacie meno a heslo.', 'error');
      $this->_fw->redirectTo('login', 'index');
    }
  }
  
  /*
   * Akcia "logout"
   * navrat: nic   
   */ 
  public function logout_action() {
    if ($this->_fw->users->logOutUser()) {
      $this->_fw->sessions->setFlashMessage('login_message', 'Bol ste odhlásený so systému.', 'success');
    } else {
      $this->_fw->sessions->setFlashMessage('login_message', 'Žiadny použivateľ nie je prihláseny. Odhlásenie nie je možné.', 'error');
    }
    $this->_fw->redirectTo('login', 'index');
  }
  
  /*
   * Zmena dat pouzivatela
   * navrat: nic   
   */   
  public function changedata_action() {
    if ($this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'change_userdata')) {
      $userdata = $this->_fw->users->getUserData();
      if (isset($_POST['meno']) && isset($_POST['heslostare']) && isset($_POST['heslo1']) && isset($_POST['heslo2'])) {
        $result = $this->_fw->users->validateDatachange($userdata['id'], $_POST['meno'], $_POST['heslostare'], $_POST['heslo1'], $_POST['heslo2']);
        if ($result === true) {
          $row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia')->load($userdata['id']);
          
          $row->setMeno($_POST['meno']);
          $row->setZmeneny(new sql_expr('CURRENT_TIMESTAMP()'));
          if ($_POST['heslostare']) {
            $row->setHeslo(sha1($_POST['heslo1']));
          }
          $row->save();
          if ($row->lastSuccess()) {
            $this->_fw->users->refreshUserData();
            $this->_fw->sessions->setFlashMessage('login_message', 'Použivatelove údaje boli úspešne zmenené.', 'success');
            $this->_fw->redirectTo('login', 'logedin');
          } else {
            $this->_fw->sessions->setFlashMessage('login_message', 'Neboli vykonané žiadne zmeny.', 'error');
            $this->_fw->callControllerAction('login', 'logedin');
          } 
        } else {
          $this->_fw->sessions->setFlashMessage('login_message', $result, 'error');
          $this->_fw->callControllerAction('login', 'logedin');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('login_message', 'Neboli odoslané všetky potrebné údaje.', 'error');
        $this->_fw->redirectTo('login', 'logedin');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('login_message', 'Nemáte povolené meniť údaje použivateľa.', 'error');
      $this->_fw->redirectTo('login', 'index');
    }
  }
  
  /*
   * Registracia
   * navrat: nic   
   */   
  public function registration_action() {
    if ($this->_fw->users->isUserLogedIn()) {
      $this->_fw->sessions->setFlashMessage('login_message', 'Prihlásený použivateľ nemá povolené vytvárať novú registráciu do systému. Najprv sa odhláste.', 'error');
      $this->_fw->redirectTo('login', 'logedin');
    }
    
    $this->_fw->templateAssign('page_title', 'Registrácia');
    $this->_fw->templateAssign('message', $this->_fw->sessions->getFlashMessage('login_message'));
    
    $this->_fw->templateAssign('registration_action', $this->_fw->makeLinkUrl('login', 'do_registration'));
    
    $this->_fw->templateDisplay('login/registration.html');
  }
  
  /*
   * Vykonanie registracie
   * navrat: nic   
   */   
  public function do_registration_action() {
    if ($this->_fw->users->isUserLogedIn()) {
      $this->_fw->sessions->setFlashMessage('login_message', 'Prihlásený použivateľ nemá povolené vytvárať novú registráciu do systému. Najprv sa odhláste.', 'error');
      $this->_fw->redirectTo('login', 'logedin');
    }
    
    if (isset($_POST['login']) && isset($_POST['meno']) && isset($_POST['heslo1']) && isset($_POST['heslo2'])) {
      $result = $this->_fw->users->validateRegistration($_POST['login'], $_POST['meno'], $_POST['heslo1'], $_POST['heslo2']);
      if ($result === true) {
        if ($this->_fw->users->createUser($_POST['login'], $_POST['meno'], $_POST['heslo1'])) {
          $this->_fw->sessions->setFlashMessage('login_message', 'Registrácia bola úspešná, môžete sa prihlásiť.', 'success');
          $this->_fw->redirectTo('login', 'index');
        } else {
          $this->_fw->sessions->setFlashMessage('login_message', 'Registrácia nebola úspešná, kvôli chybe v databázi.', 'error');
          $this->_fw->callControllerAction('login', 'registration');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('login_message', $result, 'error');
        $this->_fw->callControllerAction('login', 'registration');
      }
    } else {
    	$this->_fw->sessions->setFlashMessage('login_message', 'Neboli odoslané všetky registračné údaje.', 'error');
      $this->_fw->redirectTo('login', 'registration');
    }
  }
  
  /*
   * Úprava objednávok
   * navrat: nic   
   */   
  public function edit_order_action() {
    if ($this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'edit_order')) {
      $data = $this->_fw->getUrlParams();
      $this->_fw->loadModel('orders');
      $userdata = $this->_fw->users->getUserData();
      if ($this->_fw->orders->testUserOrder(@$userdata['id'], @$data['order_id'], $order)) {
        if ($order->getStav() == 0) {
          if (!count($_POST)) {
            $_POST['meno'] = $order->getMeno();
            $_POST['email'] = $order->getEmail();
            $_POST['adresa'] = $order->getAdresa();
            $_POST['poznamka'] = $order->getPoznamka();
          }
          $this->_fw->templateAssign('page_title', 'Úprava objednávky');
          $this->_fw->templateAssign('curent_data', $_POST);
          $this->_fw->templateAssign('order_id', $order->getId());
          $this->_fw->templateAssign('items', $order->getZoznam_tovaru());
          $this->_fw->templateAssign('order_edit_action', $this->_fw->makeLinkUrl('login', 'do_edit_order', $data));
          $this->_fw->templateAssign('message', $this->_fw->sessions->getFlashMessage('login_message'));
          $this->_fw->templateDisplay('login/edit_order.html');
        } else {
          $this->_fw->sessions->setFlashMessage('login_message', 'Nie je možné upravovať objednávku v inom stave ako "Prijatá na spracovanie"!', 'error');
          $this->_fw->redirectTo('login', 'logedin');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('login_message', 'Nemáte právo upravovať túto objednávku.', 'error');
        $this->_fw->redirectTo('login', 'logedin');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('login_message', 'Nemáte oprávnenie upravovať objednávky.', 'error');
      $this->_fw->redirectTo('login', 'index');
    }
  }
  
  /*
   * Vykonanie upravy objednavky
   * navrat: nic   
   */   
  public function do_edit_order_action() {
    if ($this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'edit_order')) {
      $data = $this->_fw->getUrlParams();
      $this->_fw->loadModel('orders');
      $userdata = $this->_fw->users->getUserData();
      if ($this->_fw->orders->testUserOrder(@$userdata['id'], @$data['order_id'], $order)) {
        if ($order->getStav() == 0) {
          $this->_fw->loadModel('cart');
          $result = $this->_fw->cart->verifyOrderData(@$_POST['meno'], @$_POST['email'], @$_POST['adresa'], 1);
          if ($result === true) {
            if ($this->_fw->orders->updateOrder($_POST, $order)) {
              $this->_fw->sessions->setFlashMessage('login_message', 'Objednávka číslo ' . $order->getId() . ' bola úspešne upravená.', 'success');
              $this->_fw->redirectTo('login', 'logedin');
            } else {
              $this->_fw->sessions->setFlashMessage('login_message', 'Objednávka nebola upravená.', 'error');
              $this->_fw->redirectTo('login', 'logedin');
            }
          } else {
            $this->_fw->sessions->setFlashMessage('login_message', $result, 'error');
            $this->_fw->callControllerAction('login', 'edit_order');
          }
        } else {
          $this->_fw->sessions->setFlashMessage('login_message', 'Nie je možné upravovať objednávku v inom stave ako "Prijatá na spracovanie"!', 'error');
          $this->_fw->redirectTo('login', 'logedin');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('login_message', 'Nemáte právo upravovať túto objednávku.', 'error');
        $this->_fw->redirectTo('login', 'logedin');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('login_message', 'Nemáte oprávnenie upravovať objednávky.', 'error');
      $this->_fw->redirectTo('login', 'index');
    }
  }
  
  public function delete_order_action() {
    if ($this->_fw->resources->testAccess($this->_fw->users->getUserGroup(), 'delete_order')) {
      $data = $this->_fw->getUrlParams();
      $this->_fw->loadModel('orders');
      $userdata = $this->_fw->users->getUserData();
      if ($this->_fw->orders->testUserOrder(@$userdata['id'], @$data['order_id'], $order)) {
        if ($order->getStav() != 1) {
          $id = $order->getId();
          if ($this->_fw->orders->deleteOrder($data, $order)) {
            $this->_fw->sessions->setFlashMessage('login_message', 'Objednávka číslo ' . $id . ' bola úspešne vymazaná.', 'success');
            $this->_fw->redirectTo('login', 'logedin');
          } else {
            $this->_fw->sessions->setFlashMessage('login_message', 'Objednávku sa nepodarilo vymazať.', 'error');
            $this->_fw->redirectTo('login', 'logedin');
          }
        } else {
          $this->_fw->sessions->setFlashMessage('login_message', 'Nie je možné vymazať objednávku v stave Spracovávaná.', 'error');
          $this->_fw->redirectTo('login', 'logedin');
        }
      } else {
        $this->_fw->sessions->setFlashMessage('login_message', 'Nemáte oprávnenie vymazávať cudzie objednávky.', 'error');
        $this->_fw->redirectTo('login', 'logedin');
      }
    } else {
      $this->_fw->sessions->setFlashMessage('login_message', 'Nemáte oprávnenie vymazávať objednávky.', 'error');
      $this->_fw->redirectTo('login', 'index');
    }
  }

}

?>