<?php

class cart {
  private $_fw; // Objekt frameworku

  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    $this->_fw = $fw;
    
    $this->_fw->loadModel('sessions');
    $this->_fw->loadModel('products');
  }
  
  /*
   * Vlozenie alebo pripocitanie produktu do kosika
   * $product_id (integer) - id produktu
   * $count (integer) - pocet kusov
   * navrat: boolean, true ak je vlozenie do kosika uspesne, inak false        
   */   
  public function addToCart($product_id, $count = 1) {
    if (intval($count) <= 0) { return false; }
    
    $cart_products = $this->_fw->sessions->restoreUserDataGroup('cart_products');
    
    $add_new = true;
    
    if (is_array($cart_products) && count($cart_products)) {
      foreach($cart_products as $key => $product_line) {
        if ($product_line['product_id'] == $product_id) {
          $cart_products[$key]['count'] += intval($count);
          $add_new = false;
          break;
        }
      }
    }
    
    if (empty($cart_products) || $add_new) {
      $cart_products[] = array(
        'product_id' => $product_id, 
        'count' => intval($count)
      );
    }
    
    $this->_fw->sessions->storeUserDataGroup('cart_products', $cart_products);
    
    return true;
  }
  
  /*
   * Navrat vsetkych produktov v kosiku
   * navrat: pole produktov, celkova suma a celkovy pocet   
   */   
  public function getCartProducts() {
    $cart_products = $this->_fw->sessions->restoreUserDataGroup('cart_products');
    
    $output = array('products' => array(), 'sum' => 0, 'count' => 0);
    $sum = 0;
    $count = 0;
    
    if (count($cart_products)) {
      foreach($cart_products as $cart_line) {
        $product = $this->_fw->products->getSingleProduct($cart_line['product_id']);
        if (count($product) > 0) {
          $output['products'][] = array(
            'product' => $product,
            'count' => $cart_line['count'],
            'price' => ($product['akcia'] ? $product['akcia'] : $product['cena']) * $cart_line['count'],
          );
          $count += $cart_line['count'];
          $sum += ($product['akcia'] ? $product['akcia'] : $product['cena']) * $cart_line['count'];
        } 
      }
    } 
    
    $output['sum'] = $sum;
    $output['count'] = $count;
    
    return $output;
  }
  
  /*
   * Zisti, ci je kosik prazdny
   * navrat: true ak je kosik prazdny   
   */   
  public function getIsCartEmpty() {
    $cart_products = $this->_fw->sessions->restoreUserDataGroup('cart_products');
    
    if (count($cart_products) == 0) {
      return true;
    }
    
    return false;
  }
  
  /*
   * Vrati jeden produkt z kosika, podla identifikatora
   * $id (integer) - id produktu, ktory sa ma najst v kosiku 
   * navrat: informacie o produkte, pocet kusov a cena     
   */   
  public function getSingleCartProduct($id) {
    $cart_products = $this->_fw->sessions->restoreUserDataGroup('cart_products');
    
    $output = array();
    
    if (count($cart_products)) {
      foreach($cart_products as $cart_line) {
        if ($cart_line['product_id'] == $id) {
          $product = $this->_fw->products->getSingleProduct($cart_line['product_id']);
          if (count($product) == 0) { $this->removeNotExistsProductFromCart($cart_line['product_id']); continue; }
          $output = array(
            'product' => $product,
            'count' => $cart_line['count'],
            'price' => ($product['akcia'] ? $product['akcia'] : $product['cena']) * $cart_line['count'],
          ); 
        }
      }
    }
    
    return $output;
  }
  
  /*
   * Upravi pocet produktov v kosiku
   * $id (integer) - id produktu v kosiku
   * $count (integer) - novy pocet kusov
   * navrat: boolean, true ak sa zmena podari, inak false         
   */   
  public function updateProductCount($id, $count) {
    if (count($this->getSingleCartProduct($id)) == 0) { return false; }
    
    $cart_products = $this->_fw->sessions->restoreUserDataGroup('cart_products');
    
    $result = false;
    
    if (count($cart_products)) {
      foreach($cart_products as $key => $product_line) {
        if ($product_line['product_id'] == $id) {
          $cart_products[$key]['count'] = intval($count);
          $result = true;
          break;
        }
      }
    } else {
      return false;
    }
    
    $this->_fw->sessions->storeUserDataGroup('cart_products', $cart_products);
    
    return $result;
  }
  
  /*
   * Zmaze produkt z kosika
   * $id (integer) - id produktu v kosiku
   * navrat: boolean, true ak bol produkt vymazany uspesne, inak false      
   */   
  public function deleteFromCart($id) {
    if (count($this->getSingleCartProduct($id)) == 0) { return false; }
    
    $cart_products = $this->_fw->sessions->restoreUserDataGroup('cart_products');
    
    $result = false;
    
    if (count($cart_products)) {
      foreach($cart_products as $key => $product_line) {
        if ($product_line['product_id'] == $id) {
          unset($cart_products[$key]);
          $result = true;
          break;
        }
      }
    } else {
      return false;
    }
    
    $this->_fw->sessions->storeUserDataGroup('cart_products', $cart_products);
    
    return $result;
  }
  
  /*
   * Skontroluje vstupne udaje formulara pre objendavku
   * $name (string) - meno a priezvisko objednavatela
   * $email (string) - emailova adresa objednavatela
   * $address (string) - postova adresa pre dorucenie tovaru
   * $platba (integer) - id zvolenej platby 
   * navrat: true, ak su informacie zadane uzivatelom v poriadku, inak vrati chybove spravy            
   */   
  public function verifyOrderData($name, $email, $address, $platba) {
    $result = array();
    
    $normalizeChars = array(
      'Š'=>'S', 'š'=>'s', 'Ð'=>'Dj','Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A',
      'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E', 'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I',
      'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U', 'Ú'=>'U',
      'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss','à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a',
      'å'=>'a', 'æ'=>'a', 'ç'=>'c', 'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i',
      'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o', 'ö'=>'o', 'ø'=>'o', 'ù'=>'u',
      'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y', 'ƒ'=>'f', 'ľ'=>'l',
    );
    
    $name = strtr($name, $normalizeChars);
    
    if (!preg_match('/^[a-zA-Z]{1,} [a-zA-Z]{1,}$/', $name)) {
      $result[] = 'Meno - prosím vložte krstné meno a priezvisko.';
    }
    
    if (mb_strlen($name, 'utf-8') > 64) {
      $result[] = 'Meno - meno môže mať najviac 64 znakov (aj s medzerou).';
    }
    
    if (!preg_match('/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,6}$/', $email)) {
      $result[] = 'Email - zadaný text nie je platná emailová adresa.';
    }
    
    if (mb_strlen($email, 'utf-8') > 128) {
      $result[] = 'Email - emailová adresa môže mať najviac 128 znakov.';
    }
    
    if (mb_strlen($address, 'utf-8') == 0) {
      $result[] = 'Adresa - toto pole je nutné vyplniť.';
    }
    
    if (intval($platba) != 1 && intval($platba) != 2 && intval($platba) != 3) {
      $result[] = 'Platba - je treba zvoliť metódu platby.';
    }
    
    return count($result) == 0 ? true : implode('<br />', $result);
  }
  
  /*
   * Vytvori objednavku (momentalne iba akoze :D)
   * $name (string) - meno a priezvisko objednavatela
   * $email (string) - emailova adresa objednavatela
   * $address (string) - postova adresa pre dorucenie tovaru
   * $platba (integer) - id zvolenej platby
   * $poznamka (string) - volitelna poznamka k objednavke      
   * navrat: boolean, true pri uspechu       
   */   
  public function storeNewOrder($name, $email, $address, $platba, $poznamka) {
    $this->_fw->loadModel('orders');
    $this->_fw->loadModel('users');
    
    $userdata = $this->_fw->users->getUserData();
    
    $produkty = $this->getCartProducts();
    
    $tovar = 'Názov tovaru';
    $tovar = $this->putTextAtPosition($tovar, 'Množstvo', 41);
    $tovar = $this->putTextAtPosition($tovar, 'Cena', 50) . "\n";
    $tovar .= str_repeat('-', 60) . "\n";
    
    foreach ($produkty['products'] as $product) {
      $line = $product['product']['nazov'];
      $line = $this->putTextAtPosition($line, ' ' . $product['count'] . 'ks ', 40);
      $line = $this->putTextAtPosition($line, $product['price'] . '&euro;', 50);
      $tovar .= $line . "\n";
    }
    $tovar .= str_repeat('-', 60) . "\n";
    $line = 'Spolu';
    $line = $this->putTextAtPosition($line, $produkty['count']. 'ks', 41);
    $line = $this->putTextAtPosition($line, $produkty['sum'] . '&euro;', 50) . "\n";
    $tovar .= $line;
    
    if ($this->_fw->orders->addNewOrder($userdata['id'], $produkty['sum'], $produkty['count'], $name, $email, $address, $platba, $poznamka, $tovar)) {
      $this->deleteCart();
      return true;
    } else {
      return false;
    }
  }
  
  /*
   * Vymaze cely kosik
   * navrat: nic   
   */   
  private function deleteCart() {
    $this->_fw->sessions->storeUserDataGroup('cart_products', array());
  }
  
  /*
   * Funkcia natiahne text tak aby novy text zacinal presne na danej pozicii
   * $text_source (string) - povodny text, za ktory sa ma doplnit novy
   * $text_to_put (string) - novy text, ktory ma byt umiestneny na danej pozicii
   * $position (integer) - pozicia, cislo >= 0
   * navrat: novy skombinovany text (string)            
   */   
  private function putTextAtPosition($text_source, $text_to_put, $position) {
    if (mb_strlen($text_source, 'utf-8') >= $position) { return $text_source . $text_to_put; }
    
    return $text_source . str_repeat(' ', $position - mb_strlen($text_source, 'utf-8')) . $text_to_put;
  }
  
  /*
   * Vyhodi s kosika produkt s danym ID (iba na interne pouzitie)
   * $id (integer) - id produktu v kosiku ktory neexistuje v DB
   * navrat: nic      
   */   
  private function removeNotExistsProductFromCart($id) {   
    $cart_products = $this->_fw->sessions->restoreUserDataGroup('cart_products');
    
    if (count($cart_products)) {
      foreach($cart_products as $key => $product_line) {
        if ($product_line['product_id'] == $id) {
          unset($cart_products[$key]);
          break;
        }
      }
    } else {
      return;
    }
    
    $this->_fw->sessions->storeUserDataGroup('cart_products', $cart_products);
  }

}

?>