<?php

class orders {
  private $_fw; // Objekt frameworku
  
  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    $this->_fw = $fw;
    
    $this->_fw->loadModel('sessions');
  }
  
  /*
   * Vrati vsetky objednavky daneho uzivatela
   * $user_id (integer) - id uzivatela
   * $filter (integer) - filtrovanie stavov, ktore sa maju zobrazit, standardne vsetky = -1
   * $pagination_id (integer) - id strankovaca      
   * navrat: pole objednavok     
   */   
  public function getOrdersByUserId($user_id, $filter = -1, $pagination_id = null) {
    $collection = $this->_fw->instanceOfTableCollection('uwa_objednavky');
    
    $collection->filterStav($filter);
    
    if (is_numeric($pagination_id)) {
      $collection->filterPagination($pagination_id);
    }
        
    return $collection->filterUzivatel($user_id)->filterOrderBy('stav', 'asc')->filterOrderBy('zmenena', 'desc')->execute()->getRowsArray('id'); 
  }
  
  /*
   * Zistik, kolko je riadkov v tabulke pre daneho uzivatela s danym filtrovanim
   * $user_id (integer) - id pouzivatela
   * $filter (integer) - cislo filtra
   * navrat: pocet riadkov v tabulke         
   */   
  public function getOrdersNumByUserId($user_id, $filter = -1) {
    $collection = $this->_fw->instanceOfTableCollection('uwa_objednavky');
    $collection->filterStav($filter)->filterUzivatel($user_id);
    
    $count = $collection->count();
    
    return intval($count);
  }
  
  /*
   * Vlozi objednavku uzivatela do systemu
   * $user_id (integer) - id uzivatela
   * $cena (integer) - cena
   * $mnozstvo (integer) - pocet kusov
   * $name (string) - meno kvoli dodaniu tovaru
   * $email (string) - email pre kontakt
   * $address (string) - adresa dodania
   * $platba (integer) - typ platby
   * $poznamka (string) - volitelna poznamka k objednavke
   * $tovar (string) - prepis objednaneho tovaru do textovej formy
   * navrat: boolean, true ak bolo vlozene uspesne, inak false                              
   */   
  public function addNewOrder($user_id, $cena, $mnozstvo, $name, $email, $address, $platba, $poznamka, $tovar) {
    $new_row = $this->_fw->instanceOfDbTable('uwa_objednavky');
    $new_row->setPouzivatel(intval($user_id))
            ->setZoznam_tovaru($tovar)
            ->setCena($cena)
            ->setMnozstvo($mnozstvo)
            ->setMeno($name)
            ->setEmail($email)
            ->setAdresa($address)
            ->setPlatba($platba)
            ->setPoznamka($poznamka);
    $new_row->setZmenena(new sql_expr('CURRENT_TIMESTAMP()'));
    $new_row->setVytvorena(new sql_expr('CURRENT_TIMESTAMP()'));
    $new_row->save();
    
    return $new_row->lastSuccess();
  }
  
  /*
   * Otestuje ci uzivatel vlastni objednavku
   * $user_id (integer) - id uzivatela
   * $order_id (integer) - id objednavky
   * $order_object (object) - referencia na objekt riadka tabulky s id $order_id   
   * navrat: boolean         
   */   
  public function testUserOrder($user_id, $order_id, &$order_object) {
    $order_object = $this->_fw->instanceOfDbTable('uwa_objednavky')->load($order_id);
    
    if($order_object->lastSuccess() && $order_object->getPouzivatel() == $user_id) {
      return true;
    }
    
    return false;
  }
  
  /*
   * Upravi objednavku dopredu vlozenu z $order_object
   * $data (array) - pole hodnot pre objednavku
   * $order_object (reference) - referencia na riadok tabulky objednavok, ktory sa ma upravit
   * navrat: boolean         
   */   
  public function updateOrder($data, &$order_object) {
    if (!(is_object($order_object) && $order_object instanceOf db_abstract && intval($order_object->getId()) != 0) && isset($data['order_id'])) {
      $order_object = $this->_fw->instanceOfDbTable('uwa_objednavky')->load($data['order_id']);
    } elseif (!isset($data['order_id'])) {
      return false;
    }
    
    if (intval($order_object->getId()) == 0) { return false; }
    
    if (isset($data['meno'])) { $order_object->setMeno($data['meno']); }
    if (isset($data['adresa'])) { $order_object->setAdresa($data['adresa']); }
    if (isset($data['email'])) { $order_object->setEmail($data['email']); }
    if (isset($data['poznamka'])) { $order_object->setPoznamka($data['poznamka']); }
    
    if (!$order_object->getAreDataChanged()) {
      return false;
    }
    
    $order_object->setZmenena(new sql_expr('CURRENT_TIMESTAMP()'));
    
    $order_object->save();
    
    return $order_object->lastSuccess();
  }
  
  public function deleteOrder($data, &$order_object) {
    if (!(is_object($order_object) && $order_object instanceOf db_abstract && intval($order_object->getId()) != 0) && isset($data['order_id'])) {
      $order_object = $this->_fw->instanceOfDbTable('uwa_objednavky')->load($data['order_id']);
    } elseif (!isset($data['order_id'])) {
      return false;
    }
    
    if (intval($order_object->getId()) == 0) { return false; }
    
    $order_object->delete();
    
    return $order_object->lastSuccess();
  }
  
  /*
   * Vrati vsetky objednavky zodpovedajuce stavu
   * $status (integer) - stav objednavky
   * $pagination_id (integer) - id strankovaca, ak ma byt tento pouzity   
   * navrat: pole dat s objednavkami      
   */   
  public function getAllOrdersByStatus($status, $pagination_id = null) {
    $collection =  $this->_fw->instanceOfTableCollection('uwa_objednavky')->filterStav($status)->filterOrderBy('stav', 'asc')->filterOrderBy('zmenena', 'desc');
    
    if (is_numeric($pagination_id)) {
    	$this->_fw->loadModel('pagination');
    	
    	$collection->filterPagination($pagination_id);
    }
    
    return $collection->execute()->getRowsArray('id');
  }
  
  /*
   * Vrati jednu objednavku
   * $id (integer) - id objednavky
   * navrat: pole hodnot objednavky      
   */   
  public function getSingleOrder($id) {
    $db = $this->_fw->instanceOfDbTable('uwa_objednavky')->load($id);
    $row = $db->data();
    if ($db->getId() != $id) { return array(); }
    $row['id'] = $row['uwa_objednavky_id'];
    return $row;
  }
  
  /*
   * Overi spravnost dat objednavky z formulara
   * $data (array) - pole dat
   * navrat: true ak je vsetko Ok, inak zoznam chyb      
   */   
  public function adminVerifyOrderData($data) {
    $result = array();
    
    $normalizeChars = array(
      'Š'=>'S', 'š'=>'s', 'Ð'=>'Dj','Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A',
      'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E', 'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I',
      'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U', 'Ú'=>'U',
      'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss','à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a',
      'å'=>'a', 'æ'=>'a', 'ç'=>'c', 'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i',
      'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o', 'ö'=>'o', 'ø'=>'o', 'ù'=>'u',
      'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y', 'ƒ'=>'f', 'ľ'=>'l',
    );
    
    if (isset($data['meno'])) {
      $name = strtr($data['meno'], $normalizeChars);
      if (!preg_match('/^[a-zA-Z]{1,} [a-zA-Z]{1,}$/', $name)) {
        $result[] = 'Meno - prosím vložte krstné meno a priezvisko.';
      }
      
      if (mb_strlen($name, 'utf-8') > 64) {
        $result[] = 'Meno - meno môže mať najviac 64 znakov (aj s medzerou).';
      }
    } else {
      $result[] = 'Meno - položka nebola odoslaná.';
    }
    
    if (isset($data['email'])) {
      if (!preg_match('/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,6}$/', $data['email'])) {
        $result[] = 'Email - zadaný text nie je platná emailová adresa.';
      }
      
      if (mb_strlen($data['email'], 'utf-8') > 128) {
        $result[] = 'Email - emailová adresa môže mať najviac 128 znakov.';
      }
    } else {
      $result[] = 'Email - položka nebola odoslaná.';
    }
    
    if (isset($data['adresa'])) {
      if (mb_strlen($data['adresa'], 'utf-8') == 0) {
        $result[] = 'Adresa - toto pole je nutné vyplniť.';
      }
    } else {
      $result[] = 'Adresa - položka nebola odoslaná.';
    }
    
    if (isset($data['cena'])) {
      if (!preg_match('/^[1-9]{1}[0-9]{0,8}$/', $data['cena'])) {
        $result[] = 'Cena - musí byť celé číslo s dĺžkou 1 až 9 cifier.';
      }
    } else {
      $result[] = 'Cena - položka nebola odoslaná.';
    }
    
    if (isset($data['mnozstvo'])) {
      if (!preg_match('/^[1-9]{1}[0-9]{0,8}$/', $data['mnozstvo'])) {
        $result[] = 'Množstvo - musí byť celé číslo s dĺžkou 1 až 9 cifier.';
      }
    } else {
      $result[] = 'Množstvo - položka nebola odoslaná.';
    }
    
    if (isset($data['stav'])) {
      if (!(is_numeric($data['stav']) && $data['stav'] >= 0 && $data['stav'] <= 2)) {
        $result[] = 'Stav - neplatná hodnota.';
      }
    } else {
      $result[] = 'Stav - položka nebola odoslaná.';
    }
    
    if (isset($data['platba'])) {
      if (!(is_numeric($data['platba']) && $data['platba'] >= 1 && $data['platba'] <= 3)) {
        $result[] = 'Platba - neplatná hodnota.';
      }
    } else {
      $result[] = 'Platba - položka nebola odoslaná.';
    }
    
    if (isset($data['pouzivatel'])) {
      $row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia')->load(intval($data['pouzivatel']));
      if ($row->getId() != $data['pouzivatel'] || intval($data['pouzivatel']) <= 0) {
        $result[] = 'Používateľ - neexistuje.';
      }
    } else {
      $result[] = 'Používateľ - položka nebola odoslaná.';
    }
    
    if (isset($data['zoznam_tovaru'])) {
      if (mb_strlen(trim($data['zoznam_tovaru']), 'utf-8') == 0) {
        $result[] = 'Súpis tovaru - položka nemôže byť prázdna.';
      }
    } else {
      $result[] = 'Súpis tovaru - položka nebola odoslaná.';
    }
    
    return count($result) == 0 ? true : implode('<br />', $result);    
  }
  
  /*
   * Vykona upravu objednavky
   * $id (integer) - id objednavky
   * $data (array) - pole dat
   * navrat: boolean         
   */   
  public function adminUpdateOrder($id, $data) {
    $row = $this->_fw->instanceOfDbTable('uwa_objednavky')->load($id);
    
    if (intval($row->getId()) == 0) { return false; }
    
    $row->setMeno($data['meno'])
        ->setEmail($data['email'])
        ->setCena($data['cena'])
        ->setMnozstvo($data['mnozstvo'])
        ->setAdresa($data['adresa'])
        ->setStav($data['stav'])
        ->setPouzivatel($data['pouzivatel'])
        ->setPlatba($data['platba'])
        ->setZoznam_tovaru($data['zoznam_tovaru'])
        ->setZmenena(new sql_expr('CURRENT_TIMESTAMP()'));
    
    if (isset($data['poznamka']) && mb_strlen(trim($data['poznamka']), 'utf-8') > 0) {
      $row->setPoznamka($data['poznamka']);
    }
    
    $row->save();
    
    return $row->lastSuccess();
  }
  
  /*
   * Vymaze vsetky uzivatelove objednavky
   * $id (integer) - id pouzivatela
   * navrat: boolean      
   */   
  public function deleteAllUserOrders($id) {
    $collection = $this->_fw->instanceOfTableCollection('uwa_objednavky');
    $collection->filterUzivatel($id)->execute();
    
    $rows = $collection->getRows();
    
    $result = true;
    
    if (count($rows)) {
      foreach($rows as $row) {
        $row->delete();
        $result = $result && $row->lastSuccess();
      }
    }
    
    return $result;
  }
}

?>