<?php

class pagination {
  private $_fw; // Objekt frameworku
  private static $data = array(); // Data pre strankovanie
   
  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    $this->_fw = $fw;
    
    $this->_fw->loadModel('sessions');
  }
  
  
  /*
   * Vytvori novy strankovac pre dannu tabulku na dany pocet riadkov
   * $table (string) - nazov tabulky na strankovanie
   * $rows_per_page (integer) - pocet riadkov na stranku
   * $additional_filter (string) - volitelne, nazov filtra, ktory sa ma na spocitanie riadkov pouzit
   * navrat: id strankovaca            
   */   
  public function createNewPagination($table, $rows_per_page, $additional_filter = '') {
    $collection = $this->_fw->instanceOfTableCollection($table);
    if ($additional_filter != '') {
      $filters = explode('|', $additional_filter);
      
      foreach ($filters as $filter) {
        $collection->$filter();
      }
    }
    
    $this->data[] = array('rpp' => $rows_per_page, 'table' => $table, 'rows' => $collection->count(), 'varname' => '');
    
    return count($this->data) - 1;
  }
  
  /*
   * Vynuti nastavenie poctu riadkov pre strankovac
   * $pagination_id (integer) - id strankovaca
   * $num_rows (integer) - pocet riadkov v tabulke
   * navrat: nic       
   */   
  public function forceSetNumRows($pagination_id, $num_rows) {
    if (!isset($this->data[$pagination_id])) { return; }
    
    if (is_numeric($num_rows) && intval($num_rows) >= 0) {
      $this->data[$pagination_id]['rows'] = intval($num_rows);
    }
  }
  
  /*
   * Rocne nastavenie unikatneho nazvu premennej pre strankovac
   * $pagination_id (integer) - id strankovaca
   * $varname (string) - nazov premennej
   * navrat: nic         
   */   
  public function setUniqueVarname($pagination_id, $varname) {
    if (!isset($this->data[$pagination_id])) { return; }
    
    if (is_string($varname)) {
      $this->data[$pagination_id]['varname'] = $varname;
    }
  }
  
  /*
   * Zadeklaruje do sablony nove premenne pre strankovac
   * $pagination_id (integer) - id strankovaca
   * $link (string) - link, kam maju viest odkazi strankovaca
   * $prefix (string) - pripadny prefix mien premennych strankovaca
   * navrat: nic            
   */   
  public function setTemplateMarkers($pagination_id, $link, $prefix = '') {
    if ($prefix != '') { $prefix .= '_'; }
    
    $min = 1;
    $max = $this->calculateMaxPage($pagination_id);
    $page = $this->getCurrentPage($pagination_id);
    
    $varname = $this->getUniqueVarname($pagination_id);
    
    $link_unique = $link . (substr($link, -1) == '/' ? '' : '/') . $varname . '/';
    
    $this->_fw->templateAssign($prefix . 'firstpage', $page > $min ? $link_unique . $min . '/' : false);
    $this->_fw->templateAssign($prefix . 'nextpage', ($page + 1 <= $max ? $link_unique . ($page + 1) . '/' : false));
    $this->_fw->templateAssign($prefix . 'prevpage', ($page - 1 >= $min ? $link_unique . ($page - 1) . '/' : false));
    $this->_fw->templateAssign($prefix . 'lastpage', $page < $max ? $link_unique . $max . '/' : false);
    
    $this->_fw->templateAssign($prefix . 'pagination_fa', $link);
    $this->_fw->templateAssign($prefix . 'select_name', $varname);
    
    $this->_fw->templateAssign($prefix . 'fromrow', $this->minRowLine($pagination_id));
    $this->_fw->templateAssign($prefix . 'torow', $this->maxRowLine($pagination_id));
    $this->_fw->templateAssign($prefix . 'totalrows', $this->totalRows($pagination_id));
    
    $pages = array();
    for ($i=$min;$i<=$max;$i++) {
      $pages[] = array('page' => $i, 'link' => $link_unique . $i . '/');
    }
    
    $this->_fw->templateAssign($prefix . 'pages', $pages);
    
    /**************************************************************************/
    
    $paginator  = '<form action="' . $link . '" method="post" name="' . $varname . '">';
    $paginator .= ($page > $min ? '<a href="' . $link_unique . $min . '/" class="pagination pagination_firstpage">&lt;&lt; Prvá stránka</a>' : '<span class="pagination">&lt;&lt; Prvá stránka</spant>');
    $paginator .= ' : ';
    $paginator .= ($page - 1 >= $min ? '<a href="' . $link_unique . ($page - 1) . '/" class="pagination pagination_prevpage">&lt; Naspäť</a>' : '<span class="pagination">&lt; Naspäť</spant>');
    $paginator .= ' : ';
    $paginator .= '<select name="' . $varname . '" size="1" class="pagination" onchange="this.form.submit();">';
    for ($i=$min;$i<=$max;$i++) {
      $paginator .= '<option value="' . $i . '"' . ($i == $page ? ' selected="selected"' : '') . '>' . $i . '</option>';
    }
    $paginator .= '</select>';
    $paginator .= ' : ';
    $paginator .= ($page + 1 <= $max ? '<a href="' . $link_unique . ($page + 1) . '/" class="pagination pagination_nextpage">Ďalej &gt;</a>' : '<span class="pagination">Ďalej &gt;</spant>');
    $paginator .= ' : ';
    $paginator .= ($page < $max ? '<a href="' . $link_unique . $max . '/" class="pagination pagination_lastpage">Posledná stránka &gt;&gt;</a>' : '<span class="pagination">Posledná stránka &gt;&gt;</spant>');
    $paginator .= '</form>';
    
    $this->_fw->templateAssign($prefix . 'paginator', $paginator);
    
    /**************************************************************************/
    
    $rowinfo  = '<span class="pagination_rowinfo">Riadky ';
    $rowinfo .= $this->minRowLine($pagination_id) . ' až ' . $this->maxRowLine($pagination_id) . ' z ' . $this->totalRows($pagination_id) . '.';
    $rowinfo .= '</span>';
    
    $this->_fw->templateAssign($prefix . 'rowinfo', $rowinfo);
  }
  
  /*
   * Zisti sucasne cislo stranky
   * $pagination_id (integer) - id strankovaca
   * navrat: cislo stranky      
   */   
  public function getCurrentPage($pagination_id) {
    $varname = $this->getUniqueVarname($pagination_id);
    
    $maxpage = $this->calculateMaxPage($pagination_id);
    
    $data = $this->_fw->getUrlParams();
    
    if (isset($_POST[$varname]) && is_numeric($_POST[$varname]) && $_POST[$varname] >= 1 && $_POST[$varname] <= $maxpage) {
      $page = $_POST[$varname];
    } elseif (isset($data[$varname]) && is_numeric($data[$varname]) && $data[$varname] >= 1 && $data[$varname] <= $maxpage) {
      $page = $data[$varname];
    } else {
      $page = $this->_fw->sessions->restoreUserDataGroup('pagination', $varname);
      if (!(is_numeric($page) && $page >= 1 && $page <= $maxpage)) {
        $page = $page > $maxpage ? $maxpage : 1 ;
      }
    }
    
    $page = $page > $maxpage ? $maxpage : $page;
    
    $full_array = $this->_fw->sessions->restoreUserDataGroup('pagination');
    $full_array[$varname] = $page;
    
    $this->_fw->sessions->storeUserDataGroup('pagination', $full_array);
    
    return $page;
  }
  
  /*
   * Vrati cislo riadku v tabulke, kde sa ma zacat vyberat dotaz
   * $pagination_id (integer) - cislo strankovaca
   * navrat: cislo prveho riadku pre aktualnu stranku      
   */   
  public function getStartLineNumber($pagination_id) {
    if (!isset($this->data[$pagination_id])) { return 0; }
    
    $curpage = $this->getCurrentPage($pagination_id);
    
    return $this->data[$pagination_id]['rpp'] * ($curpage - 1);
  }
  
  /*
   * Vrati pocet riadkov na stranku
   * $pagination_id (integer) - id strankovaca
   * navrat: pocet riadkov na stranku      
   */   
  public function getNumRowsPerPage($pagination_id) {
    if (!isset($this->data[$pagination_id])) { return 1000; }
    
    return $this->data[$pagination_id]['rpp'];
  }
  
  /*
   * Vrati cislo prveho zobrazeneho riadku (pocita sa od 1)
   * $pagination_id (integer) - id strankovaca
   * navrat: prvy zobrazeny riadok z db pocitany od 1      
   */   
  public function minRowLine($pagination_id) {
    if (!isset($this->data[$pagination_id])) { return 0; }
    
    $page = $this->getCurrentPage($pagination_id);
    
    $min = $this->data[$pagination_id]['rpp'] * ($page - 1) + 1;
    
    $min = $min < $this->data[$pagination_id]['rows'] ? $min : $this->data[$pagination_id]['rows'];
    
    return $min;
  }
  
  /*
   * Vrati cislo posledneho zobrazeneho riadku (pocita sa od 1)
   * $pagination_id (integer) - id strankovaca
   * navrat: posledny zobrazney riadok z db pocitany od 1     
   */   
  public function maxRowLine($pagination_id) {
    if (!isset($this->data[$pagination_id])) { return 0; }
    
    $page = $this->getCurrentPage($pagination_id);
    
    $max = $this->data[$pagination_id]['rpp'] * $page;
    $max = $max < $this->data[$pagination_id]['rows'] ? $max : $this->data[$pagination_id]['rows'];
    
    return $max;
  }
  
  /*
   * Vrati pocet vsetkych riadkov v tabulke
   * $pagination_id (integer) - id strankovaca
   * navrat: pocet vsetkych riadkov db      
   */   
  public function totalRows($pagination_id) {
    if (!isset($this->data[$pagination_id])) { return 0; }
    
    return $this->data[$pagination_id]['rows'];
  }
  
  /*
   * Vrati maximalnu moznu stranku
   * $pagination_id (integer) - id strankovaca
   * navrat: maximalna mozna stranka      
   */   
  private function calculateMaxPage($pagination_id) {
    if (!isset($this->data[$pagination_id])) { return 1; }
    
    $max = ceil($this->data[$pagination_id]['rows'] / $this->data[$pagination_id]['rpp']);
    
    return $max > 1 ? $max : 1;
  }
  
  /*
   * Vygeneruje unikatne meno premennej pre strankovac
   * $pagination_id (integer) - id strankovaca
   * navrat: nazov premennej      
   */   
  private function getUniqueVarname($pagination_id) {
    if (isset($this->data[$pagination_id]['varname']) && $this->data[$pagination_id]['varname'] != '') { return $this->data[$pagination_id]['varname']; }
    
    $controller = $this->_fw->getUrlSegment(0);
    $action = $this->_fw->getUrlSegment(1);
    $controller = $controller == '' ? 'index' : $controller;
    $action = $action == '' ? 'index' : $action;
  
    return $controller . '_' . $action . '_' . $pagination_id . '_page';
  }

}

?>