<?php

class products {
  private $_fw; // Objekt frameworku

  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    $this->_fw = $fw;
  }
  
  /*
   * Vrati vsetky produkty
   * $pagination_id (integer) - id strankovaca pre strankovanie produktov   
   * navrat: pole produktov   
   */   
  public function getAllProducts($pagination_id = null) {
    $this->_fw->loadModel('pagination');
    
    $collection = $this->_fw->instanceOfTableCollection('uwa_produkty');
    
    if (is_numeric($pagination_id)) {
      $collection->filterPagination($pagination_id);
    }
    
    $collection->filterOrderBy('nazov', 'asc');
    
    return $collection->execute()->getRowsArray('id');
  }
  
  /*
   * Vrati vsetky akciove produkty
   * $count (integer) - pocet produktov   
   * navrat: pole akciovych produktov   
   */   
  public function getDiscountProducts($count = 5) {
    $collection = $this->_fw->instanceOfTableCollection('uwa_produkty');
    $collection->filterOrderBy('RAND()', 'asc');
    $count = $count > 1 ? $count : 1;
    $collection->filterLimit($count,0);
    return $collection->filterAkcioveProdukty()->execute()->getRowsArray('id');
  }
  
  /*
   * Vrati jeden produkt podla ID
   * $id (integer) - id produktu
   * navrat: informacie o produkte (pole)      
   */   
  public function getSingleProduct($id) {
    $row = $this->_fw->instanceOfDbTable('uwa_produkty');
    $data = $row->load($id)->data();
    
    if (!$row->lastSuccess()) { return array(); }
    
    $data['id'] = $data['uwa_produkty_id'];
    
    return $data;
  }
  
  /*
   * Zisti, ci produkt existuje
   * $id (integer) - id produktu, ktory je treba overit
   * navrat: boolean, true ak produkt existuje, inak false      
   */   
  public function productExist($id) {
    if (count($this->getSingleProduct($id)) > 0) { return true; }
    
    return false;
  }
  
  /*
   * Overi, ci je vstup z formularu pre produkty platyn
   * $data (array) - pole informacie o produkte
   * navrat: true, ak je vsetko v poriadku, inak vrati zoznam chyb      
   */   
  public function verifyProductForm($data) {
    $result = array();
    
    if (isset($data['nazov'])) {
      if (!(mb_strlen(trim($data['nazov']), 'utf-8') > 0 && mb_strlen(trim($data['nazov']), 'utf-8') <= 255)) {
        $result[] = 'Názov - nesprávna dĺžka názvu. Povolených je 1 až 255 znakov.';
      }
    } else {
      $result[] = 'Názov - nebola odoslaná hodnota.';
    }
    
    if (isset($data['popis'])) {
      if (mb_strlen(trim($data['popis'], 'utf-8')) == 0) {
        $result[] = 'Popis - musíte zadať popis produktu.';
      }
    } else {
      $result[] = 'Popis - nebola odoslaná hodnota.';
    }
    
    if (isset($data['detail'])) {
      if (mb_strlen(trim($data['detail'], 'utf-8')) == 0) {
        $result[] = 'Detail - musíte zadať detailný popis produktu.';
      }
    } else {
      $result[] = 'Detail - nebola odoslaná hodnota.';
    }
    
    if (isset($data['cena'])) {
      if (!preg_match('/^[1-9]{1}[0-9]{0,8}$/', $data['cena'])) {
        $result[] = 'Cena - musí byť celočíselná hodnota (1 až 9 cifier).';
      }
    } else {
      $result[] = 'Cena - nebola odoslaná hodnota.';
    }
    
    if (isset($data['akcia']) && strlen($data['akcia']) > 0 && (!preg_match('/^[0-9]{1,9}$/', $data['akcia']) || $data['akcia'] >= $data['cena'])) {
      $result[] = 'Zľava - musí byť celočíselná hodnota (1 až 9 cifier) menšia ako hodnota v poli Cena.';
    }
    
    return count($result) == 0 ? true : implode('<br />', $result);
  }
  
  /*
   * Vytvori novy produkt
   * $data (array) - pole hodnot produktu z formulara
   * navrat: boolean     
   */   
  public function createNewProduct($data) {
    $row = $this->_fw->instanceOfDbTable('uwa_produkty');
    
    $row->setNazov(trim($data['nazov']))
        ->setPopis(trim($data['popis']))
        ->setDetail(trim($data['detail']))
        ->setCena(intval($data['cena']))
        ->setAkcia(0);
        
    if (isset($data['akcia']) && strlen($data['akcia']) > 0) {
      $row->setAkcia(intval($data['akcia']));
    }
    
    $row->save();
    
    return $row->lastSuccess();
  }
  
  /*
   * Vykona upravu dat produktu
   * $id (integer) - id produktu
   * $data (array) - data produktu
   * navrat: boolean         
   */   
  public function updateProduct($id, $data) {
    $row = $this->_fw->instanceOfDbTable('uwa_produkty')->load($id);
    
    if ($row->getId() !== $id) { return false; }
    
    $row->setNazov(trim($data['nazov']))
        ->setPopis(trim($data['popis']))
        ->setDetail(trim($data['detail']))
        ->setCena(intval($data['cena']))
        ->setAkcia(intval($data['akcia']));
    
    $row->save();
    
    return $row->lastSuccess();
  }
  
  /*
   * Vymaze dany produkt
   * $id (integer) - id produktu v db
   * navrat: boolean      
   */   
  public function deleteProduct($id) {
    $row = $this->_fw->instanceOfDbTable('uwa_produkty')->load($id);
    
    if ($row->getId() !== $id) { return false; }
    
    $row->delete();
    
    return $row->lastSuccess();
  }

}

?>