<?php

class resource_group {
  private $_inherite = null; // Skupina, z ktoreho sa dedi
  
  private $_resources = array(); // Povolene zdroje
  
  /*
   * Konstruktor skupiny
   * $inherite_group (instanceOf resource_group|null) - referencia na skupinu s ktorej sa maju dedit opravnenia
   * navrat: referencia na tento objekt      
   */   
  public function __construct($inherite_group = null) {
    $this->_inherite = $inherite_group;
    
    return $this;
  }
  
  /*
   * Testuje opravnenie na zdroj
   * $resource_name (string) - nazov zdroja
   * navrat: boolean, true ak je zdroj povoleny v tejto alebo dedenych skupinach, inak false      
   */   
  public function testResourceAccess($resource_name) {
    if (!is_string($resource_name)) { return false; }
    
    if (isset($this->_resources[$resource_name])) {
      return $this->_resources[$resource_name];
    } elseif(is_object($this->_inherite) && $this->_inherite instanceOf resource_group) {
      return $this->_inherite->testResourceAccess($resource_name); 
    }
    
    return false;
  }
  
  /*
   * Povoli zdroj pre tuto skupinu
   * $resource_name (string) - nazov zdroja
   * navrat: referencia na tento objekt      
   */   
  public function allowResource($resource_name) {
    if (!is_string($resource_name)) { return $this; }
    
    $this->_resources[$resource_name] = true;
    
    return $this;
  }
  
  /*
   * Zakaze zdroj pre tuto skupinu
   * $resource_name (string) - nazov zdroja
   * navrat: referencia na tento objekt      
   */
  public function disallowResource($resource_name) {
    if (!is_string($resource_name)) { return $this; }
    
    $this->_resources[$resource_name] = false;
    
    return $this;
  }
  
}

class resources {
  private $_fw; // Objekt frameworku
  
  private static $_groups; // Skupiny zdrojov

  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    $this->_fw = $fw;
  }
  
  /*
   * Vytvori novu skupinu pre zdroje
   * $group_name (string) - nazov skupiny
   * $inherite_from (string|instanceOf resource_group) - nazov alebo priama referencia na skupinu, s ktorej sa maju dedit prava na zdroje
   * navrat: referencia na vytvorenu skupinu         
   */                               
  public function createGroup($group_name, $inherite_from = null) {
    if (!isset($this->_groups[$group_name])) {
      $inherite = null;
      if (is_string($inherite_from)) {
        $inherite = isset($this->_groups[$inherite_from]) ? $this->_groups[$inherite_from] : null;
      } elseif (is_object($inherite_from) && $inherite_from instanceOf resource_group) {
        $inherite = $inherite_from;
      }
      
      $this->_groups[$group_name] = new resource_group($inherite);
    }
    
    return $this->_groups[$group_name];
  }
  
  /*
   * Vrati skupinu pokial existuje
   * $group_name (string) - nazov skupiny
   * navrat: instanceOf resource_group alebo null      
   */   
  public function getGroup($group_name) {
    return isset($this->_groups[$group_name]) ? $this->_groups[$group_name] : null;
  }
  
  /*
   * Otestuje pristup k zdroju v danej skupine
   * $group_name (string) - meno skupiny
   * $resource_name (string) - nazov zdroja
   * navrat: boolean       
   */   
  public function testAccess($group_name, $resource_name) {
    if (isset($this->_groups[$group_name])) {
      return $this->_groups[$group_name]->testResourceAccess($resource_name);
    } 
    
    return false;
  }
}

?>