<?php

class users {
  private $_fw; // Objekt frameworku
  
  /*
   * Konstruktor
   * $fw (object) - objekt frameworku
   * navrat: nic      
   */   
  public function __construct(&$fw) {
    $this->_fw = $fw;
    
    $this->_fw->loadModel('sessions');
  }
  
  /*
   * Prihlasi pouzivatela
   * $login (string) - prihlasovacie meno
   * $password (string) - heslo
   * navrat: boolean, true ak bol pouzivatel teraz prihlaseny, inak false         
   */   
  public function logInUser($login, $password) {
    if ($this->isUserLogedIn()) { return false; }
    
    $collection = $this->_fw->instanceOfTableCollection('uwa_pouzivatelia');
    $collection->filterPrihlasenie($login, $password)->execute();
    
    $data = $collection->getRowsArray('id');
    
    if (count($data) == 1) {
      $row = $data[0];
      unset($row['heslo']);
      $this->_fw->sessions->storeUserDataGroup('loged_in_user', $row);
      return true;
    }
    
    return false;
  }
  
  /*
   * Odhlasi prave prihlaseneho pouzivatela
   * navrat: true, ak bolo odhlasenie uspesne, inak false   
   */   
  public function logOutUser() {
    if (!$this->isUserLogedIn()) { return false; }
    
    $this->_fw->sessions->storeUserDataGroup('loged_in_user', array());
    
    return true;
  }
  
  /*
   * Zisti, ci je pouzivatel prihlaseny
   * navrat: boolean, true ak je uzivatel prihlaseny, inak false   
   */   
  public function isUserLogedIn() {
    if ($this->_fw->sessions->restoreUserDataGroup('loged_in_user', 'id') !== '') { return true; }
    return false;
  }
  
  /*
   * Obnovi data prihlaseneho uzivatela z databaze
   * navrat: nic   
   */   
  public function refreshUserData() {
    if ($this->isUserLogedIn()) {
      $data = $this->getUserData();
      
      $row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia')->load($data['id']);
      $new_data = $row->data();
      
      unset($new_data['heslo']);
      unset($new_data['uwa_pouzivatelia_id']);
      $new_data['id'] = $data['id'];
      
      $this->_fw->sessions->storeUserDataGroup('loged_in_user', $new_data);
    }
  }
  
  /*
   * Vrati vsetky uzivatelske data
   * navrat: pole uzivatelskych dat   
   */   
  public function getUserData() {
    if (!$this->isUserLogedIn()) { return array(); }
    
    return $this->_fw->sessions->restoreUserDataGroup('loged_in_user');
  }
  
  /*
   * Vrati vsetky objednavky pre prihlaseneho pouzivatela
   * $filter (integer) - filtrovanie stavov, ktore sa maju zobrazovat (standardne vsetky = -1) 
   * $pagination_id (integer) - id strankovaca     
   * navrat: pole objednavok   
   */   
  public function getUserOrders($filter = -1, $pagination_id = null) {
    if (!$this->isUserLogedIn()) { return array(); }
    $this->_fw->loadModel('orders');
    
    $udata = $this->getUserData();
    
    return $this->_fw->orders->getOrdersByUserId($udata['id'], $filter, $pagination_id);
  }
  
  /*
   * Vrati skupinu prihlaseneho pouzivatela
   * navrat: nazov skupiny   
   */   
  public function getUserGroup() {
    if ($this->isUserLogedIn()) {
      $data = $this->getUserData();
      return $data['skupina'];
    }
    
    return 'anonymous';
  }
  
  /*
   * Overuje platnost vstupnych hodnot pri registracii
   * $login (string) - meno pouzivatela
   * $meno (string) - krstne meno a priezvisko
   * $heslo1 (string) - heslo
   * $heslo2 (string) - kontrola hesla
   * navrat: true ak je vsetko v poriadku, inak string - zoznam chyb               
   */   
  public function validateRegistration($login, $meno, $heslo1, $heslo2) {
    $result = array();
    
    $normalizeChars = array(
      'Š'=>'S', 'š'=>'s', 'Ð'=>'Dj','Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A',
      'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E', 'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I',
      'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U', 'Ú'=>'U',
      'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss','à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a',
      'å'=>'a', 'æ'=>'a', 'ç'=>'c', 'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i',
      'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o', 'ö'=>'o', 'ø'=>'o', 'ù'=>'u',
      'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y', 'ƒ'=>'f', 'ľ'=>'l',
    );
    
    if (!(is_string($login) && trim($login) == $login && preg_match('/^[a-zA-Z0-9_-]{6,64}$/', $login))) {
      $result[] = 'Prihlasovacie meno - povinný rozsah je 6 až 64 znakov. Povolená znaky sú znaky: a-z A-Z 0-9 _ -';
    }
    
    $collection = $this->_fw->instanceOfTableCollection('uwa_pouzivatelia');
    $collection->filterUsername($login)->execute();
    
    if (count($collection->getRows()) > 0) {
      $result[] = 'Prihlasovacie meno - zadané meno už existuje.';
    }
    
    $meno = strtr($meno, $normalizeChars);
    
    if (!preg_match('/^[a-zA-Z]{1,} [a-zA-Z]{1,}$/', $meno)) {
      $result[] = 'Krstné meno a priezvisko - prosím vložte krstné meno a priezvisko.';
    }
    
    if (!(mb_strlen($heslo1, 'utf-8') >= 6 && mb_strlen($heslo2, 'utf-8') >= 6 && $heslo1 === $heslo2)) {
      $result[] = 'Heslo - heslo musí mať aspoň 6 znakov a obe zadané heslá sa musia zhodovať.';
    }
    
    return count($result) == 0 ? true : implode('<br />', $result);
  }
  
  /* 
   * Vytvorenie noveho pouzivatela
   * $login (string) - prihlasovacie meno
   * $meno (string) - meno a priezvisko
   * $heslo (string) - heslo
   * navrat: uspesnost akcie, true ak bol pouzivatel vytvoreny, inak false           
   */   
  public function createUser($login, $meno, $heslo) {
    $new_row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia');
    
    $new_row->setLogin($login)->setMeno($meno)->setHeslo(sha1($heslo))->setSkupina('user')->setIp_adresa(getenv('REMOTE_ADDR'))
            ->setVytvoreny(new sql_expr('CURRENT_TIMESTAMP()'))->setZmeneny(new sql_expr('CURRENT_TIMESTAMP()'))->save();
    
    return $new_row->lastSuccess();
  }
  
  /*
   * Overenie platnosti udajov pre zmenu pouzivatelskeho mena alebo hesla
   * $id (integer) - id pouzivatela
   * $meno (string) - meno a priezvisko
   * $heslostare (string) - stare heslo
   * $heslo1 (string) - nove heslo
   * $heslo2 (string) - nove heslo pre kontrolu
   * navrat: true ak je vsetko v poriadku, inak zoznam chyb (ako string)                  
   */   
  public function validateDatachange($id, $meno, $heslostare, $heslo1, $heslo2) {
    $result = array();
    
    $normalizeChars = array(
      'Š'=>'S', 'š'=>'s', 'Ð'=>'Dj','Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A',
      'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E', 'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I',
      'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U', 'Ú'=>'U',
      'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss','à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a',
      'å'=>'a', 'æ'=>'a', 'ç'=>'c', 'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i',
      'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o', 'ö'=>'o', 'ø'=>'o', 'ù'=>'u',
      'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y', 'ƒ'=>'f', 'ľ'=>'l',
    );
    
    $meno = strtr($meno, $normalizeChars);
    
    if (!preg_match('/^[a-zA-Z]{1,} [a-zA-Z]{1,}$/', $meno)) {
      $result[] = 'Krstné meno a priezvisko - prosím vložte krstné meno a priezvisko.';
    }
    
    if (strlen($heslostare) > 0 || strlen($heslo1) > 0 || strlen($heslo2) > 0) {
    
      if (!(mb_strlen($heslostare) > 0)) {
        $result[] = 'Staré heslo - je treba vyplniť staré heslo kvôli kontrole.';
      } 
      
      $row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia')->load($id);
      
      if ($row->getHeslo() !== sha1($heslostare)) {
        $result[] = 'Staré heslo - zadané heslo je nesprávne.';
      }
    
      if (!(mb_strlen($heslo1, 'utf-8') >= 6 && mb_strlen($heslo2, 'utf-8') >= 6 && $heslo1 === $heslo2)) {
        $result[] = 'Heslo - heslo musí mať aspoň 6 znakov a obe zadané heslá sa musia zhodovať.';
      }
    
    }
    
    return count($result) == 0 ? true : implode('<br />', $result);
  }
  
  /*
   * Vrati zoznam vsetkych pouzivatelov
   * navrat: pole pouzivatelov   
   */   
  public function getAllUsers() {
    $collection = $this->_fw->instanceOfTableCollection('uwa_pouzivatelia');
    $collection->filterOrderBy('meno', 'asc')->execute();
    
    return $collection->getRowsArray('id');
  }
  
  /*
   * Skontroluje vstupne hodnoty z formulara na vytvorenie/upravu pouzivatela
   * $id (integer) - id uzivatela (pri upravovani)
   * $data (array) - pole udajov o pouzivatelovi
   * $create (boolean) - urcuje, ci ide o vytvaranie pouzivatela alebo upravu (vtedy false)
   * navrat: true, ak je vsetko v poriadku, inak zoznam chyb            
   */   
  public function validateAdminForm($id, $data, $create = true) {
    $result = array();
    
    $normalizeChars = array(
      'Š'=>'S', 'š'=>'s', 'Ð'=>'Dj','Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A',
      'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E', 'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I',
      'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U', 'Ú'=>'U',
      'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss','à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a',
      'å'=>'a', 'æ'=>'a', 'ç'=>'c', 'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i',
      'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o', 'ö'=>'o', 'ø'=>'o', 'ù'=>'u',
      'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y', 'ƒ'=>'f', 'ľ'=>'l',
    );
    
    if (isset($data['meno'])) {
      $meno = strtr($data['meno'], $normalizeChars);
      
      if (!preg_match('/^[a-zA-Z]{1,} [a-zA-Z]{1,}$/', $meno)) {
        $result[] = 'Meno - prosím vložte krstné meno a priezvisko.';
      }
    } else {
      $result[] = 'Meno - hodnota nebola odoslaná.';
    }
    
    if (isset($data['login'])) {
      $checklogin = false;
      if ($create == false) {
        $row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia')->load($id);
        if ($row->getLogin() != $data['login']) { $checklogin = true; }
      }
      
      if ($create == true) { $checklogin = true; }
      
      if (!(is_string($data['login']) && trim($data['login']) == $data['login'] && preg_match('/^[a-zA-Z0-9_-]{6,64}$/', $data['login']))) {
        $result[] = 'Login - povinný rozsah je 6 až 64 znakov. Povolená znaky sú znaky: a-z A-Z 0-9 _ -';
      }
      
      if ($checklogin) {
        $collection = $this->_fw->instanceOfTableCollection('uwa_pouzivatelia');
        $collection->filterUsername($data['login'])->execute();
        
        if (count($collection->getRows()) > 0) {
          $result[] = 'Login - zadané meno už existuje.';
        }
      }
    } else {
      $result[] = 'Login - hodnota nebola odoslaná.';
    }
    
    if (isset($data['skupina'])) {
      if (!($data['skupina'] == 'user' || $data['skupina'] == 'editor' || $data['skupina'] == 'admin')) {
        $result[] = 'Skupina - neplatná hodnota.';
      }
    } else {
      $result[] = 'Skupina - hodnota nebola odoslaná.';
    }
    
    if ($create == true) {    
      if (isset($data['heslo1']) && isset($data['heslo2'])) {
        if (!(mb_strlen($data['heslo1'], 'utf-8') >= 6 && mb_strlen($data['heslo2'], 'utf-8') >= 6 && $data['heslo1'] === $data['heslo2'])) {
          $result[] = 'Heslo - heslo musí mať aspoň 6 znakov a obe zadané heslá sa musia zhodovať.';
        }
      } else {
        $result[] = 'Heslo - hodnota nebola odoslaná.';
      }
    } else {
      if (isset($data['heslo1']) && isset($data['heslo2'])) {
        if (mb_strlen($data['heslo1'], 'utf-8') > 0 || mb_strlen($data['heslo2'], 'utf-8') > 0) {
          if (!(mb_strlen($data['heslo1'], 'utf-8') >= 6 && mb_strlen($data['heslo2'], 'utf-8') >= 6 && $data['heslo1'] === $data['heslo2'])) {
            $result[] = 'Heslo - heslo musí mať aspoň 6 znakov a obe zadané heslá sa musia zhodovať.';
          }
        }
      } else {
        $result[] = 'Heslo - hodnota nebola odoslaná.';
      }
    }
    
    return count($result) == 0 ? true : implode('<br />', $result);
  }
  
  /* 
   * Vytvorenie noveho pouzivatela
   * $data (array) - pole udajov
   * navrat: uspesnost akcie, true ak bol pouzivatel vytvoreny, inak false           
   */   
  public function adminCreateUser($data) {
    $new_row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia');
    
    $new_row->setLogin($data['login'])->setMeno($data['meno'])->setHeslo(sha1($data['heslo1']))->setSkupina($data['skupina'])
            ->setIp_adresa(getenv('REMOTE_ADDR'))->setVytvoreny(new sql_expr('CURRENT_TIMESTAMP()'))->setZmeneny(new sql_expr('CURRENT_TIMESTAMP()'))->save();
    
    return $new_row->lastSuccess();
  }
  
  /*
   * Vymaze pouzivatela a vsetky jeho objednavky
   * $id (integer) - id pouzivatela
   * navrat: boolean      
   */   
  public function deleteUser($id) {
    $row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia')->load($id);
    
    if (intval($row->getId()) == 0) { return false; }
    
    $this->_fw->loadModel('orders');
    
    $result = $this->_fw->orders->deleteAllUserOrders($id);
    
    $row->delete();
    
    $result = $result && $row->lastSuccess();
    
    return $result;
  }
  
  /*
   * Vrati uzivatelove data z db
   * $id (integer) - id pouzivatela
   * navrat: pole dat pouzivatela      
   */   
  public function getSingleUser($id) {
    $row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia')->load($id);
    
    $data = array();
    
    if (intval($row->getId()) > 0) {
      $data = $row->data();
      $data['id'] = $data['uwa_pouzivatelia_id'];
    }
    
    return $data; 
  }
  
  /*
   * Upravi pouzivatela
   * $id (integer) - id pouzivatela
   * $data (array) - pole dat pouzivatela
   * navrat: boolean         
   */   
  public function editUser($id, $data) {
    $row = $this->_fw->instanceOfDbTable('uwa_pouzivatelia')->load($id);
    
    if (intval($row->getId()) == 0) { return false; }
    
    $userdata = $this->getUserData();
    
    $row->setMeno($data['meno'])->setLogin($data['login']);
    if ($userdata['id'] != $id) { $row->setSkupina($data['skupina']); }
    
    if (isset($data['heslo1']) && isset($data['heslo2']) && $data['heslo1'] == $data['heslo2'] && strlen($data['heslo1']) > 0 && strlen($data['heslo2']) > 0) {
      $row->setHeslo(sha1($data['heslo1']));
    }
    
    $row->setZmeneny(new sql_expr('CURRENT_TIMESTAMP()'));
    $row->save();
    
    $this->refreshUserData();
    
    return $row->lastSuccess();
  }
}

?>