<?php

class collection_abstract {

  protected $_fw; // Objekt frameworku
  protected $_link; // Link na databazove spojenie
  protected $_tablename; // Nazov tabulky
  protected $_idfield; // Nazov primarneho kluca tabulky
  
  private $_rows = array(); // Riadky tabulky
  
  protected $_where = ''; // Dodatocne podmienky vo WHERE klauzule dotazu
  protected $_orderby = ''; // Dodatocne triedenie zoznamu
  protected $_limit = ''; // Dodatocne limitovanie zoznamu
  
  private $_last_query = ''; // Posledne spustena query
  private $_last_error = ''; // Posledne chyby v query
  private $_last_success = true; // Informacia o poslednom uspechu/neuspechu

  /*
   * Konstruktor
   * $fw (object reference) - referencia na objekt frameworku
   * $link (resource) - link na databazove spojenie
   * navrat: nic         
   */   
  public function __construct(&$fw, $link) {
    $this->_fw = $fw;
    $this->_link = $link;
    $this->_tablename = $this->getTableNameByClassName();
    $this->_idfield = $this->_tablename . '_id';
  }
  
  /*
   * Vykona SELECT dotaz nad tabulkou
   * navrat: referencia na tento objekt   
   */   
  public function execute() {
    $sql = 'SELECT * FROM `' . $this->_tablename . '`';
    
    if ($this->_where != '') {
      $sql .= ' WHERE ' . $this->_where;
    }
    
    if ($this->_orderby != '') {
      $sql .= ' ORDER BY ' . $this->_orderby;
    } 
    
    if ($this->_limit != '') {
      $sql .= ' LIMIT ' . $this->_limit;
    } 
    
    $sql .= ';';
    
    $this->_last_query = $sql;
    $result = mysql_query($sql, $this->_link);
    $this->_fw->incQueries();
    
    if ($result) {
      $this->_last_success = true;
      $this->_last_error = '';
      $this->_rows = array();
      while ($r = mysql_fetch_assoc($result)) {
        $this->_rows[] = $this->_fw->instanceOfDbTable($this->_tablename, $r);
      }
      
      mysql_free_result($result);
    } else {
      $this->_last_success = false;
      $this->_last_error = mysql_errno($this->_link) . ': ' . mysql_error($this->_link);
    }
    
    return $this;
  }
  
  /*
   * Vrati pocet riadkov tabulky
   * navrat: pocet riadkov tabulky alebo false pri chybe   
   */   
  public function count() {
    $sql = 'SELECT COUNT(' . $this->_idfield . ') as count_num_rows FROM ' . $this->_tablename;
    
    if ($this->_where != '') {
      $sql .= ' WHERE ' . $this->_where;
    }
    
    $sql .= ';';
    
    $this->_last_query = $sql;
    $result = mysql_query($sql, $this->_link);
    $this->_fw->incQueries();
    
    if ($result) {
      $this->_last_success = true;
      $this->_last_error = '';
      
      $row = mysql_fetch_assoc($result);
      return intval($row['count_num_rows']);
    } else {
      $this->_last_success = false;
      $this->_last_error = mysql_errno($this->_link) . ': ' . mysql_error($this->_link);
      
      return false;
    }
  }
  
  /*
   * Vrati posledne spustenu query
   * navrat: text query   
   */   
  public function lastQuery() {
    return $this->_last_query;
  }
  
  /*
   * Vrati poslednu mysql chybu
   * navrat: text mysql chyby   
   */   
  public function lastError() {
    return $this->_last_error;
  }
  
  /*
   * Vrati informaciu o uspechu/neuspechu poslednej operacie
   * navrat: true/false informacia o poslednej operacia   
   */   
  public function lastSuccess() {
    return $this->_last_success;
  }
  
  /*
   * Vrati vsetky nacitane riadky ako pole objektov riadkov tabulky
   * navrat: pole objektov   
   */   
  public function getRows() {
    return $this->_rows;
  }
  
  /*
   * Vrati vsetky nacitane riadky ako asociativne pole
   * navrat: pole riadkov tabulky   
   */   
  public function getRowsArray($renameIdTo = null) {
    $output = array();
    
    if (count($this->_rows)) {
      foreach($this->_rows as $row) {
        $line = $row->data();
        if (is_string($renameIdTo)) {
          $line[$renameIdTo] = $line[$this->_idfield];
        }
        
        if (method_exists($this, 'getRowsArrayHook')) {
          $line = $this->getRowsArrayHook($line, $row);
        }
        
        $output[] = $line;
      }
    }
    
    return $output;
  }
  
  /*
   * Nastavi filter na vyber vsetkych zaznamov, ktorych primarny kluc ma jednu z hodnot zoznamu id-ciek
   * $ids (string|array) - ciarkou oddelovany zoznam alebo pole id-ciek
   * navrat: referencia na tento objekt      
   */   
  public function filterSelectIds($ids) {
    if (is_string($ids)) {
      if (strlen(trim($ids))) {
        $ids = explode(',', $ids);
      } else {
        $ids = array();
      }
    }
    
    if (is_array($ids) && count($ids)) {
      $list = '';
      $sep = '';
      foreach($ids as $id) {
        $list .= $sep . intval(trim($id));
        $sep = ',';
      }
      
      $this->_where .= strlen($this->_where) == 0 ? '' : ' AND ';
      $this->_where .= $this->_idfield . ' IN (' . $list . ')';
    }
    
    return $this;
  }
  
  /*
   * Nastavi order by polozku do dotazu na vyber
   * $col (string) - stlpec v tabulke, podla ktoreho sa bude zotriedovat zoznam
   * $direction (string) - smer zoradenia ASC/DESC
   * navrat: referencia na tento objekt         
   */   
  public function filterOrderBy($col, $direction = 'ASC') {
    $this->_orderby .= (strlen($this->_orderby) ? ', ' : '') . $col . ' ' . (strtoupper($direction) == 'ASC' ? 'ASC' : 'DESC');
    
    return $this;
  }
  
  /*
   * Nastavi limit vyhladanych riadkov
   * $count (integer) - pocet, ktory sa ma vyhladat
   * $from (integer) - prvy riadok, od ktoreho sa ma hladat
   * navrat: referencia na tento objekt         
   */   
  public function filterLimit($count, $from = 0) {
    $this->_limit = $from . ', ' . $count;
    
    return $this;
  }
  
  /*
   * Nastavi limit vyhladavania riadkov podla modelu strankovania
   * $pagination_id (integer) - id strankovaca
   * navrat: referencia na tento objekt      
   */   
  public function filterPagination($pagination_id) {
    $this->_fw->loadModel('pagination');
    
    $this->_limit = $this->_fw->pagination->getStartLineNumber($pagination_id) . ', ' . $this->_fw->pagination->getNumRowsPerPage($pagination_id);
    
    return $this;
  }
  
  /*
   * Vyresetuje vsetky pripadne modifikacie dotazu
   * navrat: nic   
   */   
  public function resetFilters() {
    $this->_where = '';
    $this->_orderby = '';
    $this->_limit = '';
    
    return $this;
  }
  
  /*
   * Nastavi vlastny nazov pre tabulku, pripadne nazov jej primarneho kluca
   * $tablename (string) - nazov tabulky
   * $id (string) - nazov stlpca primarneho kluca
   * navrat: referencia na tento objekt        
   */   
  public function customTablename($tablename, $id = null) {
    $this->_tablename = $tablename;
    $this->_idfield = $id == null ? $this->_tablename . '_id' : $id;
    
    return $this;
  }
  
  /*
   * Zisti nazov tabulky z nazvu triedy
   * navrat: nazov tabulky   
   */   
  private function getTableNameByClassName() {
    $classname = get_class($this);
    $segments = explode('_', $classname);
    if (count($segments) > 1) {
      unset($segments[count($segments) - 1]);
      $classname = implode('_', $segments);
    } else {
      die('<strong>Fatal error:</strong> Invalid class name for db collection model!');
    }
    return strtolower($classname);
  }
}

?>