<?php

class sql_expr {
  private $_value = ''; // SQL vyraz, napriklad funkcia
  
  /*
   * Konstruktor
   * $value (string|numeric) - cislo alebo retazec, napriklad mysql funkcia
   * navrat: nic      
   */     
  public function __construct($value) {
    if (is_string($value) || is_numeric($value)) {
      $this->_value = $value;
    } else { $this->_value = ''; }
  }
  
  /*
   * Vrati sql vyraz ulozeny v objekte
   */   
  public function getValue() {
    return $this->_value;
  }
}

class db_abstract {

  protected $_link; // Linka na databazove spojenie
  protected $_fw; // Objekt frameworku
  protected $_tablename; // Nazov tabulky
  protected $_idfield = 'id'; // Nazov primarneho kluca tabulky
  
  private $_data = array(); // Pracovne data pre riadok tabulky
  private $_original = array(); // Povodne data v riadku tabulky
  private $_columns = array(); // Konfiguracia stlpcov tabulky
  private $_last_query = ''; // Posledne spustena query
  private $_last_error = ''; // Posledne chyby v query
  private $_last_success = true; // Informacia o poslednom uspechu/neuspechu

  /*
   * Konstruktor
   * $fw (class reference) - referencia na objekt frameworku
   * $link (resource) - link na databazove spojenie
   * $data (array) - vychodzie data pre riadok tabulky       
   */   
  public function __construct(&$fw, $link, $data = array()) {
    $this->_fw = $fw;
    $this->_link = $link;
    $this->_tablename = $this->getTableNameByClassName();
    $this->_idfield = $this->_tablename . '_id';
    $this->reloadColumnsDefinitionOfTable();
    
    if (is_array($data) && count($data)) {
      foreach($data as $key => $value) {
        if (in_array($key, $this->_columns)) {
          $this->_data[$key] = stripslashes($value);
          $this->_original[$key] = stripslashes($value); 
        }
      }
    }
  }
  
  /*
   * Nacita data z databazy podla primarneho kluca
   * $id (integer) - cislo zaznamu v databazovej tabulke
   * navrat: referencia na tento objekt      
   */
  public function load($id) {
    $sql = 'SELECT * FROM `' . $this->_tablename . '` WHERE ' . $this->_idfield . ' = ' . intval($id) . ';';
    $this->_last_query = $sql;
    $result = mysql_query($sql, $this->_link);
    $this->_fw->incQueries();
    if ($result === false) {
      $this->_last_error = mysql_errno($this->_link) . ': ' . mysql_error($this->_link);
      $this->_last_success = false;
      $this->_original = array();
      $this->_data = array();
    } else {
      if (mysql_num_rows($result)) {
        $this->_last_error = '';
        $this->_last_success = true;
        $row = mysql_fetch_assoc($result);
        $this->_original = array();
        $this->_data = array();
        foreach ($row as $key => $value) {
          $this->_original[$key] = stripslashes($value);
          $this->_data[$key] = stripslashes($value);
        }
      } else {
        $this->_last_error = '';
        $this->_last_success = false;
        $this->_original = array();
        $this->_data = array();
      }
      mysql_free_result($result);
    }
    return $this;
  }
  
  /*
   * Ulozenie/prepisanie dat do databazy (pokial data existuju a sucasne su odlisne od originalu)
   * navrat: referencia na tento objekt  
   */   
  public function save() {
    $this->_last_success = false;
    $this->_last_error = '';
    if ($this->_data !== $this->_original && count($this->_data)) {
      if (isset($this->_data[$this->_idfield])) {
        $sql = 'UPDATE `' . $this->_tablename . '` ';
      } else {
        $sql = 'INSERT INTO `' . $this->_tablename . '` ';
      }
      
      $sql .= 'SET ';
      
      $sql_set = array();
      
      foreach($this->_data as $variable => $value) {
        if (is_object($value) && $value instanceOf sql_expr) {
          $sql_set[] = $variable . ' = ' . $value->getValue();
        } else {
          $sql_set[] = $variable . ' = ' . (is_string($value) ? '"' . addslashes($value) . '" ' : $value . ' ');
        }
      }
      
      $sql .= implode(', ', $sql_set);
      
      if (isset($this->_data[$this->_idfield])) {
        $sql .= 'WHERE ' . $this->_idfield . ' = ' . intval($this->_data[$this->_idfield]);
      }
      
      if (method_exists($this, 'saveHook')) {
        $sql .= ' ' . $this->saveHook();
      }
      
      if (isset($this->_data[$this->_idfield])) {
        $sql .= ' LIMIT 1';
      }
      
      $sql .= ';';
      
      $this->_last_query = $sql;
      $result = mysql_query($sql, $this->_link);
      $this->_fw->incQueries();
      
      if ($result && !isset($this->_data[$this->_idfield])) {
        $this->_data[$this->_idfield] = mysql_insert_id($this->_link);
      } 
      if ($result) {
        $this->_last_error = '';
        $this->_last_success = true;
      } else {
        $this->_last_error = mysql_errno($this->_link) . ': ' . mysql_error($this->_link);
        $this->_last_success = false;
      }
      
      $this->_original = $this->_data;
    }
    
    return $this;
  }
  
  /*
   * Vymaze z tabulky riadok
   * navrat: referencia na tento objekt   
   */   
  public function delete() {
    $this->_last_error = '';
    $this->_last_success = false;
    if (isset($this->_data[$this->_idfield])) {
      $sql = 'DELETE FROM `' . $this->_tablename . '` WHERE ' . $this->_idfield . ' = ' . intval($this->_data[$this->_idfield]) . ' LIMIT 1;';
      $this->_last_query = $sql;
      $result = mysql_query($sql, $this->_link);
      $this->_fw->incQueries();
      
      if ($result) {
        $this->_last_error = '';
        $this->_last_success = true;
        $this->_original = array();
        $this->_data = array();
      } else {
        $this->_last_error = mysql_errno($this->_link) . ': ' . mysql_error($this->_link);
        $this->_last_success = false;
      }
    }
    
    return $this;
  }
  
  /*
   * Nastavenie a citanie dat
   * $name (null|string) - nazov stlpca, ktory sa ma vypisat/zmenit
   * $value (null|string|numeric) - hodnota pre danu bunku stlpca
   * navrat: hodnota bunky tabulky / cely obsah tabulky ako asociativne pole / referencia na tento objekt         
   */   
  public function data($name = null, $value = null) {
    if ($name === null && $value === null) {
      return $this->_data;
    } elseif ($name !== null && $value === null) {
      if ($name == 'id') { $name = $this->_idfield; }
      
      if (isset($this->_data[$name])) {
        return $this->_data[$name];
      } 
      
      return ''; 
    } else {
      if ($name == 'id') { $name = $this->_idfield; }
      
      if (in_array($name, $this->_columns)) {
        $this->_data[$name] = $value;
      }
      
      return $this;
    }
  }
  
  /*
   * Virtualizacia metod s prefixom set a get (na citanie a nastavovanie dat)
   * $method_name (string) - meno metody
   * $arguments (array) - pole argumentov metody
   * navrat: obsah bunky tabulky (pre get...) alebo referencia na tento objekt (pre set...)        
   */   
  public function __call($method_name, $arguments) {
    if (substr($method_name, 0, 3) == 'get') {
      $name = strtolower(substr($method_name, 3));
      return $this->data($name);
    } elseif (substr($method_name, 0, 3) == 'set') {
      $name = strtolower(substr($method_name, 3));
      if (count($arguments) != 1) {
        throw new exception('<strong>Warning:</strong> ' . $method_name . ' or ' . get_class($this) . ' needs one argument!<br />');
        return $this;
      }
      return $this->data($name, $arguments[0]);
    }
  }
  
  /*
   * Nastavi vlastny nazov pre tabulku, pripadne nazov jej primarneho kluca
   * $tablename (string) - nazov tabulky
   * $id (string) - nazov stlpca primarneho kluca
   * navrat: referencia na tento objekt        
   */   
  public function customTablename($tablename, $id = null) {
    $this->_tablename = $tablename;
    $this->_idfield = $id == null ? $this->_tablename . '_id' : $id;
    $this->reloadColumnsDefinitionOfTable();
    
    $this->_data = array();
    $this->_original = array();
    
    return $this;
  }
  
  /*
   * Vrati posledne spustenu query
   * navrat: text query   
   */   
  public function lastQuery() {
    return $this->_last_query;
  }
  
  /*
   * Vrati poslednu mysql chybu
   * navrat: text mysql chyby   
   */   
  public function lastError() {
    return $this->_last_error;
  }
  
  /*
   * Vrati informaciu o uspechu/neuspechu poslednej operacie
   * navrat: true/false informacia o poslednej operacia   
   */   
  public function lastSuccess() {
    return $this->_last_success;
  }
  
  /*
   * Vrati informaciu, ci boli data zmenene a teda je mozne ich zapisat
   * navrat: boolean   
   */   
  public function getAreDataChanged() {
    return $this->_data !== $this->_original;
  }
  
  /*
   * Zisti nazov tabulky z nazvu triedy
   * navrat: nazov tabulky   
   */   
  private function getTableNameByClassName() {
    $classname = get_class($this);
    $segments = explode('_', $classname);
    if (count($segments) > 1) {
      unset($segments[count($segments) - 1]);
      $classname = implode('_', $segments);
    } else {
      die('<strong>Fatal error:</strong> Invalid class name for db row model!');
    }
    return strtolower($classname);
  }
  
  /*
   * Nacita alebo obnovy uz nacitanu konfiguraciu stlpcov v tabulke
   * navrat: nic   
   */   
  private function reloadColumnsDefinitionOfTable() {
    if (!isset($GLOBALS['db_columns'][$this->_tablename])) {
      $result = mysql_query('SHOW FULL COLUMNS FROM ' . $this->_tablename . ';', $this->_link);
      $this->_fw->incQueries();
      if ($result === false) {
        die('<strong>Fatal error:</strong> Unknown table ' . $this->_tablename . '!');
      }
      
      $this->_columns = array();
      
      while($r = mysql_fetch_assoc($result)) {
        $this->_columns[] = $r['Field'];
      }
      $GLOBALS['db_columns'][$this->_tablename] = $this->_columns;
    } else {
      $this->_columns = $GLOBALS['db_columns'][$this->_tablename]; 
    }
  }

}

?>