<?php

class swift {
  
  //private $compiler;
  public $config;
  private $asgvalues;
  private $swiftvalues;
  private $file_level = 0;
  private $debug_data = '<h1>Swift debug info</h1>';
    
  private function getContent($template_file) {
    $file_path = $this->config['tpldir'] . ($this->config['tpldir'][strlen($this->config['tpldir']) - 1] == '/' ? $template_file : '/' . $template_file);
    
    if (!file_exists($file_path)) {
      return '';
    }
    
    $hash = sha1($file_path) . '.php';
    
    $compiled_path = $this->config['cmpdir'] . ($this->config['cmpdir'][strlen($this->config['cmpdir']) - 1] == '/' ? $hash : '/' . $hash);
    
    if (!file_exists($compiled_path)) {
      $compiler = new swift_compiler($this);
      $compiler->assign_config($this->config);
      $compiler->compile($file_path, $compiled_path);
    }
    
    $this->addDebugFileInfo($file_path, $compiled_path);
    
    include($compiled_path);
  }
  
  public function __construct() {
    //$this->compiler = new swift_compiler($this);
    $this->config = array(
      'tpldir' => 'templates',
      'cmpdir' => 'compiled',
      'swfdir' => 'swift',
      'tagleft' => '{',
      'tagright' => '}',
      'debug' => false,
      'allowPHP' => false,
    );
    $this->asgvalues['swift'] = array(
      'post' => $_POST,
      'get' => $_GET,
      'request' => $_REQUEST,
      'session' => isset($_SESSION) ? $_SESSION : array(),
      'cookie' => $_COOKIE,
      'server' => $_SERVER,
      'version' => '0.9.1 RC1',
      'tpldir' => $this->config['tpldir'],
      'cmpdir' => $this->config['cmpdir'],
      'swfdir' => $this->config['swfdir'],
      'tagleft' => $this->config['tagleft'],
      'tagright' => $this->config['tagright'],
      'debug' => $this->config['debug'],
      'allowPHP' => $this->config['allowPHP'],
    );
  }
  
  public function setTags($left_tag, $right_tag) {
    if (strlen(trim($left_tag)) > 0 && strlen(trim($right_tag)) > 0)
    $this->config['tagleft'] = $left_tag;
    $this->config['tagright'] = $right_tag;
    $this->asgvalues['swift']['tagleft'] = $left_tag;
    $this->asgvalues['swift']['tagright'] = $right_tag;
  }
  
  public function setTPLdir($dir) {
    if (file_exists($dir)) {
      $this->config['tpldir'] = $dir;
      $this->asgvalues['swift']['tpldir'] = $dir;
    } else {
      die('<strong>Swift error:</strong> Path "' . $dir . '" not found!');
    }
  }
  
  public function setCMPdir($dir) {
    if (file_exists($dir)) {
      $this->config['cmpdir'] = $dir;
      $this->asgvalues['swift']['cmpdir'] = $dir;
    } else {
      die('<strong>Swift error:</strong> Path "' . $dir . '" not found!');
    }
  }
  
  public function setSWFdir($dir) {
    if (file_exists($dir)) {
      $this->config['swfdir'] = $dir;
      $this->asgvalues['swift']['swfdir'] = $dir;
    } else {
      die('<strong>Swift error:</strong> Path "' . $dir . '" not found!');
    }
  }
  
  public function setDebuging($boolean = true) {
    $this->config['debug'] = $boolean == true;
    $this->asgvalues['swift']['debug'] = $boolean == true;
  }
  
  public function setPHPallowed($boolean = true) {
    $this->config['allowPHP'] = $boolean == true;
    $this->asgvalues['swift']['allowPHP'] = $boolean == true;
  }
  
  public function __destruct() {
    //$this->compiler->__destruct();
  }
  
  public function assign($variable, $value) {
    if ($variable != 'swift') {
      $this->asgvalues[$variable] = $value;
    }
  }
  
  public function assign_array($array) {
    foreach ($array as $k => $v) {
      if (!is_numeric($k)) {
        $this->assign($k, $v);
      }
    }
  }
  
  public function unassign($variable) {
    if ($variable == 'swift') { return; }
    if (!isset($this->asgvalues[$variable])) {
      unset($this->asgvalues[$variable]);
    }
  }
  
  public function parse($template_file) {
    $this->file_level++;
    $content = '';
    ob_start();
    $this->getContent($template_file);
    $content = ob_get_contents();
    ob_clean();
    $this->file_level--;
    if ($this->file_level == 0 && $this->config['debug']) {
      $this->showDebug();
    }
    return $content;
  }
  
  public function clear_all_compiled() {
    $dir = opendir($this->config['cmpdir']);
    while (false !== ($file = readdir($dir))) {
      list($name, $ext) = explode('.', $file, 2);
      if (strlen($name) == 40 && $ext == 'php') {
        $file = $this->config['cmpdir'] . ($this->config['cmpdir'][strlen($this->config['cmpdir']) - 1] == '/' ? $file : '/' . $file);
        unlink($file);
      }
    }
  }
  
  private function showDebug() {
    $this->debug_array_level = 0;
    
    if (!function_exists('writeVariablesArray')) {
      function writeVariablesArray($array) {
        $output = '';
        if (is_array($array)) {
          $output .= '<table>';
          foreach ($array as $key => $value) {
            $output .= '<tr>';
            $output .= '<td class="td1"><strong>[Array]</strong>&nbsp;' . $key . '</td>';
            $output .= '<td class="td2">' . writeVariablesArray($value) . '</td>';
            $output .= '</tr>';
          }
          $output .= '</table>';
        } else {
          $type = 'unknow';
          if (is_bool($array)) { $type = 'boolean'; $array = $array == true ? 'true' : 'false'; }
          else if (is_int($array)) { $type = 'integer'; }
          else if (is_string($array)) { $type = 'string'; }
          else if (is_double($array)) { $type = 'double'; }
          else if (is_float($array)) { $type = 'float'; }
          else if (is_resource($array)) { $type = 'resource'; }
          else if (is_object($array)) { $type = 'object'; }
          $output .= '<strong>['.$type.']</strong>&nbsp;' . nl2br(htmlspecialchars($array));
        }
        return $output;
      }
    }
    
    $this->debug_data .= '<hr />';
    $this->debug_data .= writeVariablesArray($this->asgvalues);
    $this->debug_data .= '<hr />';
    
    $this->debug_data .= '<style type="text/css">';
    $this->debug_data .= '* { margin: 0px; padding: 0px; font-family: "Courier"; background-color: #AAAAFF; font-size: 8px; }';
    $this->debug_data .= 'strong { color: blue; }';
    $this->debug_data .= 'table { width: 100%; border-collapse: collapse; border: 1px solid #5555FF; border-bottom: 0px; border-right: 0px; }';
    $this->debug_data .= 'td { vertical-align: top; border-bottom: 1px solid #5555FF; border-right: 1px solid #5555FF; }';
    $this->debug_data .= '.td1 { width: 20%; }';
    $this->debug_data .= '</style>';
    
    echo '<script type="text/javascript">';
    echo 'var swift_debug_window = window.open("", "swift_debug_window", "widht=800,height=600,resizable=yes,scrollbars=yes");';
    $debug_data = str_replace('"', '\"', $this->debug_data);
    //$debug_data = str_replace("'", "\'", $debug_data);
    for ($i=0;$i<32;$i++) {
      $debug_data = str_replace(chr($i), '', $debug_data);
    } 
    echo 'swift_debug_window.document.write("'.$debug_data.'");';
    echo '</script>';
    
    $this->debug_data = '<h1>Swift debug info</h1>';
  }
  
  private function addDebugFileInfo($template_file, $compiled_file) {
    if (!$this->config['debug']) { return; }
    $md5 = md5($template_file);
    $this->debug_data .= '<hr />';
    $this->debug_data .= '<strong>Template file: </strong><a href="#tpl_' . $md5 . '" onclick="document.getElementById(\'tpl_' . $md5 . '\').style.display=\'block\'; return false;">'. $template_file . '</a><br />';
    $this->debug_data .= '<strong>Template size: </strong>'. filesize($template_file) . 'bytes <br />';
    $this->debug_data .= '<div style="display: none;" id="tpl_' . $md5 . '">' . highlight_file($template_file, true) . '<br /><br /><a href="#top" onclick="document.getElementById(\'tpl_' . $md5 . '\').style.display=\'none\'; return false;"><strong>[CLOSE]</strong></a></div>';
    $this->debug_data .= '<strong>Compiled file: </strong><a href="#cpl_' . $md5 . '" onclick="document.getElementById(\'cpl_' . $md5 . '\').style.display=\'block\'; return false;">'. $compiled_file . '</a><br />';
    $this->debug_data .= '<strong>Compiled size: </strong>'. filesize($compiled_file) . 'bytes <br />';
    $this->debug_data .= '<div style="display: none;" id="cpl_' . $md5 . '">' . highlight_file($compiled_file, true) . '<br /><br /><a href="#top" onclick="document.getElementById(\'cpl_' . $md5 . '\').style.display=\'none\'; return false;"><strong>[CLOSE]</strong></a></div>';
  }
}

class swift_compiler {
  
  private $config;
  private $file;
  private $line;
  private $swift;
  private $stack;
  private $require_list = array();
  
  public function __construct(&$swift) {
    $config = array();
    $this->swift = $swift;
    $this->stack = new swift_stack();
  }
  
  public function assign_config($config) {
    $this->config = $config;
  }
  
  public function compile($file, $cfile) {
    $bc_file = $this->file;
    $this->file = $file;
    $bc_line = $this->line;
    $this->line = 1;
    
    $content = '';
    $f = fopen($file, 'r');
    while (!feof($f)) {
      $content .= fgets($f);
    }
    fclose($f);
    
    if ($this->config['allowPHP'] == false) {
      $content = $this->delete_PHP_content($content);
    }
    
    $content = $this->compile_content($content);
    
    $f = fopen($cfile, 'w');
    fputs($f, $content);
    fclose($f);
    
    $this->file = $bc_file;
    $this->line = $bc_line;
  }
  
  private function delete_PHP_content($content) {
    
    $output = '';
    $found = 0;
    for($i=0;$i<strlen($content);$i++) {
      if (substr($content, $i, 2) == '<?') { $found++; $i+=2; }
      if (substr($content, $i, 2) == '?>' && $found > 0) { $found--; $i+=2; }
      if ($found == 0) { $output .= $content[$i]; }
    }
    
    return $output;
  }
  
  private function compile_content($content) {
    $ocontent = $content;
    $content = '';
    
    $ltl = strlen($this->config['tagleft']);
    $rtl = strlen($this->config['tagright']);
    $lt = $this->config['tagleft'];
    $rt = $this->config['tagright'];
    
    for ($pos=0;$pos<strlen($ocontent);$pos++) {
      if (substr($ocontent, $pos, $ltl) == $lt) {
        $pos_end = strpos($ocontent, $rt, $pos+$ltl);
        $code = substr($ocontent, $pos + $ltl, $pos_end - $pos - $ltl);
        $pos = $pos_end + $rtl -1;
        $reparse = false;
        $content .= $this->examine_code($code, $reparse);
        if ($reparse) {
          $pos = $pos - strlen($code) - $rtl;
        }
      } else {
        $content .= $ocontent[$pos];
      }
      
      if ($ocontent[$pos] == "\n") { $this->line++; }
    }
    
    $content = $this->build_require_list() . "\n" . $content;
    
    return $content;
  }
  
  private function examine_code($code, &$reparse) {
    $lt = $this->config['tagleft'];
    $rt = $this->config['tagright'];
    
    // Code is variable output
    if ($code[0] == '$' && $this->stack->get() != 'skip') {
      return $this->output_variable($code);
    }
    // Code is comment
    else if (substr($code, 0, 2) == '**') {
      return '';
    }
    // Code is block output
    else if (substr($code, 0, 5) == 'block' && $this->stack->get() != 'skip') {
      $this->stack->add('block');
      return $this->output_block($code);
    }
    // Code is end of block output
    else if (substr($code, 0, 6) == '/block' && $this->stack->get() != 'skip') {
      $stack = $this->stack->get();
      if ($stack == 'block') {
        $this->stack->delete();
        return '<?php }} ?>';
      }
      if ($stack == 'emptyblock') {
        $this->stack->delete();
        return '<?php } ?>';
      }
      $this->error('Unexpected <strong>/block</strong> tag here!');
    }
    // Code is emptyblock output
    else if (substr($code, 0, 10) == 'emptyblock' && $this->stack->get() != 'skip') {
      $stack = $this->stack->get();
      if ($stack != 'block') {
        $this->error('Unexpected <strong>emptyblock</strong> tag here!');
      }
      $this->stack->delete();
      $this->stack->add('emptyblock');
      return '<?php }} else { ?>';
    }
    // Code is loop output
    else if (substr($code, 0, 4) == 'loop' && $this->stack->get() != 'skip') {
      $this->stack->add('loop');
      return $this->output_loop($code);
    }
    // Code is end of loop output
    else if (substr($code, 0, 5) == '/loop' && $this->stack->get() != 'skip') {
      if ($this->stack->get() != 'loop') {
        $this->error('Unexpected <strong>/loop</strong> tag here!');
        die();
      }
      $this->stack->delete();
      return '<?php } ?>';
    }
    // Code is include of new template
    else if (substr($code, 0, 7) == 'include' && $this->stack->get() != 'skip') {
      return $this->output_include($code);
    }
    // Code is cycler output
    else if (substr($code, 0, 6) == 'cycler' && $this->stack->get() != 'skip') {
      return $this->output_cycler($code);
    }
    // Code is IF condition
    else if (substr($code, 0, 2) == 'if' && $this->stack->get() != 'skip') {
      $this->stack->add('if');
      return '<?php ' . $this->output_if($code) . ' ?>';
    }
    // Code is end of IF condition
    else if (substr($code, 0, 3) == '/if' && $this->stack->get() != 'skip') {
      $stack = $this->stack->get();
      if ($stack == 'if' || $stack == 'else') {
        $this->stack->delete();
        return '<?php } ?>';
      } else {
        $this->error('Unexpected <strong>/if</strong> tag here!');
      }
    }
    // Code is ELSEIF condition
    else if (substr($code, 0, 6) == 'elseif' && $this->stack->get() != 'skip') {
      $stack = $this->stack->get();
      if ($stack == 'else') {
        $this->error('<strong>elseif</strong> tag can\'t be used after <strong>else</strong> tag!');
      } else if ($stack != 'if') {
        $this->error('<strong>elseif</strong> tag can\'t be used out of <strong>if</strong> tag statement!');
      } else {
        return '<?php } else ' . $this->output_if($code) . ' ?>';
      }
    }
    // Code is ELSE statement
    else if (substr($code, 0, 4) == 'else' && $this->stack->get() != 'skip') {
      $stack = $this->stack->get();
      if ($stack == 'else') { $this->error('<strong>else</strong> tag can\'t be used more than once after <strong>if/elseif</strong> tag.'); }
      else if ($stack != 'if') { $this->error('<strong>else</strong> tag can\'t be used alone.'); }
      else {
        $this->stack->delete();
        $this->stack->add('else');
        return '<?php } else { ?>';
      }
    }
    // Code is in-template variable assign
    else if (substr($code, 0, 6) == 'assign' && $this->stack->get() != 'skip') {
      return $this->output_assign($code);
    }
    // Code is function type plugin
    else if ((substr($code, 0, 2) == 'f=' || substr($code, 0, 9) == 'function=' || substr($code, 0, 1) == '=') && $this->stack->get() != 'skip') {
      $cut_pos = strpos($code, '=', 0);
      $code = substr($code, $cut_pos, strlen($code) - $cut_pos);
      return $this->output_function($code);
    }
    // Code is output of left delimiter for swift tag
    else if (substr($code, 0, 6) == 'ldelim' && $this->stack->get() != 'skip') {
      return $lt;
    }
    // Code is output of right delimiter for swift tag
    else if (substr($code, 0, 6) == 'rdelim' && $this->stack->get() != 'skip') {
      return $rt;
    }
    // Code is start of skiping all of swift tags (except /skip tag)
    else if (substr($code, 0, 4) == 'skip') {
      $this->stack->add('skip');
      return '';
    }
    // Code is end of skiping all of swift tags
    else if (substr($code, 0, 5) == '/skip') {
      if ($this->stack->get() == 'skip') {
        $this->stack->delete();
      } else {
        $this->error('<strong>/skip</strong> tag can\'t be used without opening tag <strong>skip</strong>.');
      }
      return '';
    }
    
    $reparse = true;
    
    return $lt;
  }
  
  // Function for create a PHP variant echo of $code variable
  private function output_variable($code, $strict_php = false, $default = '') {
    if ($code[0] == '$') {
      $code = substr($code, 1, strlen($code) - 1);
    }
    
    $modif_array = $this->determine_modifiers($code);
    
    $code = $modif_array[0];
    
    if (strpos($code, '[') !== false) {
    
      $n = 0;
      $parts = array();
      
      $start = strpos($code, '[');
      
      $var = substr($code, 0, $start);
      
      $c = '';
      for ($i=$start;$i<strlen($code);$i++) {
        if ($code[$i] == '[') {
          $n++;
          if ($n > 1) { $c .= '['; }
        } else if ($code[$i] == ']') {
          $n--;
          if ($n > 0) { $c .= ']'; }
        } else {
          $c .= $code[$i];
        }
        if ($n == 0) {
          $parts[] = $c;
          $c = '';
        }
      }
      
      $code = '\'' . $var . '\'';
      
      foreach ($parts as $part) {
        if ($part[0] == '$') {
          $code .= '][';
          $code .= $this->output_variable($part, true);
        } else {
          $code .= '][';
          $code .= '\'' . $part . '\'';
        }
      }
    } else {
      $code = '\'' . $code . '\'';
    }
    
    $build = '';
    if (!$strict_php) {
      $build = '<?php echo ';
    }
    if (count($modif_array) > 1) {
      $build .= $this->build_modifier_code($modif_array, '(isset($this->asgvalues[' . $code . ']) ? $this->asgvalues[' . $code . '] : \'' . $default . '\')', 1);
    } else {
      $build .= '(isset($this->asgvalues[' . $code . ']) ? $this->asgvalues[' . $code . '] : \'' . $default . '\')';
    }
    if (!$strict_php) {
      $build .= '; ?>';
    }
    
    return $build;
  }
  
  // This function create a foreach cycle in PHP output
  private function output_block($code) {
    if (substr($code, 0, 5) == 'block') {
      $code = trim(substr($code, 5, strlen($code) - 5));
    }
    
    $params = $this->getParameters($code);
    
    if (isset($params['use']) && isset($params['row'])) {
      if ($params['use']['variable'] == false) {
        $this->error('Parameter "use" must be variable!');
      }
      if ($params['row']['variable'] == true) {
        $this->error('Parameter "row" must be string!');
      }
      
      $use = $this->output_variable($params['use']['value'], true);
      
      $out = '<?php ';
      $out .= 'if (is_array(' . $use . ') && count(' . $use . ') > 0) { ';
      $out .= '$this->asgvalues[\'swift\'][\'block\'][\'' . $params['row']['value'] . '\'][\'line\'] = 0;';
      $out .= '$this->asgvalues[\'swift\'][\'block\'][\'' . $params['row']['value'] . '\'][\'size\'] = count(' . $use . ');';
      $out .= 'foreach (' . $use . ' as $this->asgvalues[\'swift\'][\'block\'][\'' . $params['row']['value'] . '\'][\'key\'] => $this->asgvalues[\'' . $params['row']['value'] . '\']) { ';
      $out .= '$this->asgvalues[\'swift\'][\'block\'][\'' . $params['row']['value'] . '\'][\'line\']++;';
      $out .= ' ?>';
      return $out;
    } else {
      $this->error('Missing some parameters for <strong>block</strong>!');
    }
  }
  
  private function output_loop($code) {
    if (substr($code, 0, 4) == 'loop') {
      $code = trim(substr($code, 4, strlen($code) - 4));
    }
    
    $params = $this->getParameters($code);
    
    if (!isset($params['name'])) { $this->error('Parameter "name" is needed!'); }
    if ($params['name']['variable'] == true) { $this->error('Parameter "name" must be string!'); }
    if (isset($params['loop']) && $params['loop']['variable'] == false) { $this->error('Parameter "loop" must be variable!'); }
    
    $var_prefix = '$this->asgvalues[\'swift\'][\'loop\'][\'' . $params['name']['value'] . '\']';
    $var_name = '$this->asgvalues[\'' . $params['name']['value'] . '\']';
    
    $loop = (isset($params['loop']) ? 'count(' . $this->output_variable($params['loop']['value'], true) . ')' : $var_prefix . '[\'max\']');
    
    $out = '<?php ';
    $out .= $var_prefix . '[\'from\'] = ' . (isset($params['from']) ? ($params['from']['variable'] == true ? $this->output_variable($params['from']['value'], true) : 'intval(\'' . $params['from']['value'] . '\')' ) : '0') . ';';
    $out .= $var_prefix . '[\'max\'] = ' . (isset($params['max']) ? ($params['max']['variable'] == true ? $this->output_variable($params['max']['value'], true) : 'intval(\'' . $params['max']['value'] . '\')' ) : '0') . ';';
    $out .= $var_prefix . '[\'step\'] = ' . (isset($params['step']) ? ($params['step']['variable'] == true ? $this->output_variable($params['step']['value'], true) : 'intval(\'' . $params['step']['value'] . '\')' ) : '1') . ';';
    $out .= 'for (' . $var_name . '=' . $var_prefix . '[\'from\'];' . $var_name . ' < ' . $loop . ';' . $var_name . '+=' . $var_prefix . '[\'step\']) {';
    $out .= 'if (' . $var_prefix . '[\'max\'] == ' . $var_name . ') { break; }';
    $out .= $var_prefix . '[\'line\'] = ' . $var_name . ' + 1;';
    $out .= ' ?>';
    
    return $out;
  }
  
  private function output_include($code) {
    if (substr($code, 0, 7) == 'include') {
      $code = trim(substr($code, 7, strlen($code) - 7));
    }
    
    $params = $this->getParameters($code);
    
    if (isset($params['file'])) {
      $useasign = false;
      if (isset($params['assign'])) {
        if ($params['assign']['variable'] == false) {
          $useasign = true;
        } else {
          $this->error('Parameter "assign" must be string!');
        }
      }
      
      $file = ($params['file']['variable']) ? $this->output_variable($params['file']['value'], true) : '\'' . $params['file']['value'] . '\'';
      
      $out = '<?php ';
      $out .= ($useasign ? '$this->asgvalues[\'' . $params['assign']['value'] . '\'] = ' : 'echo ') . '$this->parse(' . $file . ');';
      $out .= ' ?>';
      
      return $out;
    } else {
      $this->error('<strong>Include</strong> file not specified!');
    }
  }
  
  private function output_cycler($code) {
    if (substr($code, 0, 6) == 'cycler') {
      $code = trim(substr($code, 6, strlen($code) - 6));
    }
    
    $params = $this->getParameters($code);
    
    if (!isset($params['values'])) { $this->error('<strong>Cycler</strong> values are not specified!'); return ''; }
    if (!isset($params['delimiter'])) { $params['delimiter']['value'] = ','; $params['delimiter']['variable'] = false; }
    
    $values = $params['values']['variable'] == true ? $this->output_variable($params['values']['value'], true) : '\'' . $params['values']['value'] . '\'';
    $delimiter = $params['delimiter']['variable'] == true ? $this->output_variable($params['delimiter']['value'], true, ',') : '\'' . $params['delimiter']['value'] . '\'';
    $cname = sha1($this->file . $this->line);
    
    $output  = '<?php ';
    $output .= '$cycler_array = explode(' . $delimiter . ', ' . $values . ');';
    $output .= '$cycler_items = count($cycler_array);';
    $output .= '$cycler_counter = isset($this->asgvalues[\'swift\'][\'cycler\'][\'' . $cname . '\']) ? $this->asgvalues[\'swift\'][\'cycler\'][\'' . $cname . '\'] : 0;';
    $output .= '$cycler_value = $cycler_array[$cycler_counter % $cycler_items];';
    $output .= '$this->asgvalues[\'swift\'][\'cycler\'][\'' . $cname . '\'] = $cycler_counter + 1;';
    
    if (isset($params['assignto'])) {
      if ($params['assignto']['variable'] == true) {
        $this->error('<strong>assignto</strong> parameter can\'t be a variable!'); return '';
      }
      $output .= '$this->asgvalues[\'' . $params['assignto']['value'] . '\'] = $cycler_value;';
    } else {
      $output .= 'echo $cycler_value;';
    }
    $output .= ' ?>';
    
    return $output;
  }
  
  private function output_if($code) {
    if (substr($code, 0, 2) == 'if') {
      $code = trim(substr($code, 2, strlen($code) - 2));
    }
    if (substr($code, 0, 6) == 'elseif') {
      $code = trim(substr($code, 6, strlen($code) - 6));
    }
    
    $output  = 'if (';
    
    $intext = false;
    $invar = false;
    $subpart = '';
    $conditions = '';
    $code = ' ' . $code . ' ';
    for ($i=0;$i<strlen($code);$i++) {
      if ($code[$i] == '$' && $intext == false && $invar == false) {
        $subpart = '';
        $invar = true;
      }
      if ($code[$i] == '"' && $code[$i-1] != "\\") {
        $intext = !$intext;
      }
      if (($code[$i] == ' ' || $code[$i] == ')' || $code[$i] == '&' || $code[$i] == '|' || $code[$i] == '=' || $code[$i] == '!' || $code[$i] == '<' || $code[$i] == '>') && $invar == true && $intext == false) {
        $invar = false;
        //echo $subpart;
        $conditions .= $this->output_variable($subpart, true);
      }
      if ($invar == true) {
        $subpart .= $code[$i];
      } else {
        $conditions .= $code[$i];
      }
      //echo '"' . $code[$i] . '"('.$invar.'/'.$intext.') ';
    }
    
    $output .= trim($conditions);
    $output .= ') {';
    
    return $output;
  }
  
  private function output_assign($code) {
    if (substr($code, 0, 6) == 'assign') {
      $code = trim(substr($code, 6, strlen($code) - 6));
    }
    
    $params = $this->getParameters($code);
    
    if (!isset($params['to'])) { $this->error('<strong>assign</strong> tag needs <strong>to</strong> parameter.'); return; }
    if (!isset($params['value'])) { $this->error('<strong>assign</strong> tag needs <strong>value</strong> parameter.'); return; }
    if ($params['to']['variable'] == true) { $this->error('<strong>to</strong> parameter must be a string.'); return; }
    
    $output  = '<?php ';
    $output .= '$this->asgvalues[\'' . $params['to']['value'] . '\'] = ';
    if ($params['value']['variable'] == true) {
      $output .= $this->output_variable($params['value']['value'], true);
    } else {
      $output .= '"' . $params['value']['value'] . '"';
    }
    $output .= '; ?>';
    
    return $output;
  }
  
  private function output_function($code, $direct_output = true) {
    if ($code[0] == '=') { $code = substr($code, 1, strlen($code) - 1); }
    $fname_pos = strpos($code, ' ', 0);
    $fname_pos = $fname_pos > 0 ? $fname_pos : strlen($code);
    $fname = substr($code, 0, $fname_pos);
    $code = substr($code, $fname_pos + 1, strlen($code) - 1);
    $params = $this->getParameters($code);
    
    $fname = 'swift_function_' . $fname;
    
    $params_array = 'array(';
    
    foreach ($params as $variable => $data) {
      $params_array .= '\'' . $variable . '\' => ' . ($data['variable'] == 1 ? $this->output_variable($data['value'], true) : '\'' . str_replace("'", "\'", $data['value']) . '\'') . ', ';
    }
    
    $params_array .= ')';
    
    $this->add_require_file($fname . '.php');
    if ($direct_output)
      return '<? echo ' . $fname . '($this, ' . $params_array . '); ?>';
    return $fname . '($this, ' . $params_array . ')';
  }
  
  // This function determine and return parameters from gived string
  private function getParameters($code) {
    $out = array();
    
    $pname = ''; $pvalue = '';
    
    $start = 0;
    $end = strpos($code, '=', $start);
    
    
    // Parameter can be variable or string
    while ($end !== false) {
      $pname = trim(substr($code, $start, $end - $start));
      $quote = false;
      $newstart = strlen($code);
      for ($i=$end + 1;$i<strlen($code);$i++) {
        if ($code[$i] == ' ' && !$quote) {
          $newstart = $i;
          break;
        }
        if ($code[$i] == '"' && $code[$i-1] != "\\") {
          $quote = !$quote;
        }
        $pvalue .= $code[$i]; 
      }
      $var = true;
      $pvalue = trim($pvalue);
      if ($pvalue[0] == '"') {
        $pvalue = substr($pvalue, 1, strlen($pvalue) - 2);
        $var = false;
      }
      if ($var == true && ($pvalue == 'true' || $pvalue == 'false')) {
        $var = false;
        $pvalue = $pvalue == 'true' ? 1 : 0;
      }
      $start = $newstart;
      
      $out[$pname] = array('variable' => $var, 'value' => $pvalue);
      
      $pname = '';
      $pvalue = '';
      $end = strpos($code, '=', $newstart);
    }
    
    return $out;
  }
  
  private function error($message, $warning = false) {
    $mgs = ($warning ? '<strong>Swift warning:</strong> ' : '<strong>Swift error:</strong> ') . $message . ' <strong>' . $this->file . '</strong> (line ' . $this->line . ')<br />';
    if ($warning) {
      echo $mgs;
    } else {
      die($mgs);
    }
  }
  
  //This function get a $code of variable output and create array with wariable name and all modifiers for this variable
  private function determine_modifiers($code) {
    // Character ; means, that this is separator for next modifier
    // Character : means, that this is separator for next parameter
    $output = array();
    $cur_mod = 0; // Current modifier ID, 0 means that this is variable code
    $in_text = false; // Means that the current character is inside text block or not
    $cur_modif_code = '';
    for ($i=0; $i<strlen($code); $i++) {
      if ($code[$i] == '"' && $code[$i-1] != "\\") { $in_text = !$in_text; }
      else if ($code[$i] == '"' && $code[$i-1] == "\\") { $cur_modif_code[strlen($cur_modif_code) - 1] = '&'; $cur_modif_code .= 'quot;'; }
      else if ($code[$i] == ';' && !$in_text) {
        $output[$cur_mod] = $this->determine_modifier_parameters($cur_modif_code, $cur_mod);
        $cur_modif_code = '';
        $cur_mod++; 
      } else {
        $cur_modif_code .= $code[$i];
      }
    }
    $output[$cur_mod] = $this->determine_modifier_parameters($cur_modif_code, $cur_mod);
    
    return $output;
  }
  
  private function determine_modifier_parameters($code, $modID) {
    if ($modID == 0)
      return $code;
    
    $in_text = false;
    $in_pts = 0;
    $output = array();
    $cur_code = '';
    $cur_param = 0;
    for ($i=0;$i<strlen($code);$i++) {
      if ($code[$i] == '"') { $in_text = !$in_text; }
      else if ( $code[$i] == '(' && !$in_text ) { $in_pts++; $cur_code .= '('; }
      else if ( $code[$i] == ')' && !$in_text ) { $in_pts--; $cur_code .= ')'; }
      else if ( $code[$i] == ':' && !$in_text && $in_pts == 0 ) {
        $output[$cur_param] = $this->determine_modifier_parameter_type($cur_code, $cur_param);
        $cur_param++;
        $cur_code = '';
      } else {
        $cur_code .= $code[$i];
      }
    }
    $output[$cur_param] = $this->determine_modifier_parameter_type($cur_code, $cur_param);
    
    return $output;
  }
  
  private function determine_modifier_parameter_type($code, $cur_param) {
    if ($cur_param == 0) 
      return 'swift_modifier_' . $code;
    
    // Check if parameter is a variable, then parse is and return php output
    if ($code[0] == '(' && $code[strlen($code) - 1] == ')' && $code[1] == '$') {
      return $this->output_variable(substr($code, 2, strlen($code) - 3), true);
    }
    return '"' . $code . '"';
  }
  
  private function build_modifier_code($modif_array = array(), $value = '', $index = 1) {
    if (isset($modif_array[$index])) {
      $code = $modif_array[$index];
      $this->add_require_file($code[0] . '.php');
      $pre = $code[0] . '($this, ';
      $post = '';
      for ($i=1;$i<count($code);$i++) {
        $post .= ', ' . $code[$i];
      }
      $post .= ')';
      $post;
      $between = $value;
      if (count($modif_array) - 1 > $index) {
        $between = $this->build_modifier_code($modif_array, $value, $index+1);
      }
      return $pre . $between . $post;
    } else { 
      return $value;
    }
  }
  
  private function add_require_file($file) {
    if (!isset($this->require_list[$file])) {
      $this->require_list[$file] = true;
    }
  }
  
  private function build_require_list() {
    $output = array();
    foreach($this->require_list as $file => $status) {
      if ($status) {
        list($function, $ext) = explode('.', $file);
        $output[] = 'if (!function_exists("' . $function . '")) { require_once("' . $this->swift->config['swfdir'] . '/plugins/' . $file . '"); }';
      }
    }
    if (count($output) > 0) {
      return '<?php ' . "\n" . implode("\n", $output) . "\n" . '?>';
    } 
    return '';
  }
}

class swift_stack {
  
  private $data;
  
  public function __construct() {
    $this->data = array();
  }
  
  public function is_empty() {
    return count($this->data) == 0 ? true : false;
  }
  
  public function add($value) {
    $this->data[count($this->data)] = $value;
  }
  
  public function get() {
    if (!$this->is_empty()) {
      return $this->data[count($this->data) - 1];
    }
    return false;
  }
  
  public function delete() {
    if (!$this->is_empty()) {
      unset($this->data[count($this->data) - 1]);
    }
  }
}

?>