<?php
// Author: Jan Boldis

/* function=html_checkbox
  parameters:
    type = can have two values, "checkboxes" of "radiobuttons", define which type of input element is used
    values = array of values and captions for each generated checkbox/radio button
    selected = array of selected items or string value of selected item
    value_key = the key of array <values> which contain datas for value paramater for each chechbox/radio button
    caption_key = the key of array <values> which contain datas for labels of each checkbox/radio button
    num_cols = integer value, the number of cols for generated table of checkboxes/radio buttons
    name = the name of checkboxes/radio buttons
    css_prefix = the prefix for classes of created elements
*/
function swift_function_html_checkbox(&$swift, $params = array()) {
  $type = (isset($params['type']) ? ($params['type'] == 'radiobuttons' ? 'radiobuttons' : 'checkboxes') : 'checkboxes');
  $values = (isset($params['values']) ? $params['values'] : array());
  $selected = (isset($params['selected']) ? $params['selected'] : NULL);
  $value_key = (isset($params['value_key']) ? $params['value_key'] : 'value');
  $caption_key = (isset($params['caption_key']) ? $params['caption_key'] : 'caption');
  $num_cols = (isset($params['num_cols']) ? (intval($params['num_cols']) > 0 ? intval($params['num_cols']) : 1) : 1);
  $name = (isset($params['name']) ? $params['name'] : 'swift_' . ($type == 'checkboxes' ? 'checkboxes' : 'radiobuttons'));
  $css_prefix = (isset($params['css_prefix']) ? $params['css_prefix'] : 'swift');
  
  $default_array = array( 'TBLROW_CLASS' => '' );
  
  $tbl_row = '<tr class="TBLROW_CLASS">';
  for ($i=1;$i<=$num_cols;$i++) {
    $tbl_row .= '<td class="TBLCOL' . $i . '_CLASS">TBLCOL' . $i . '_CONTENT</td>';
    $default_array['TBLCOL' . $i . '_CLASS'] = '';
    $default_array['TBLCOL' . $i . '_CONTENT'] = '';
  }
  $tbl_row .= '</tr>';
  
  if ($type == 'checkboxes') {
    $html_name = $name . '[]';
    $html_type = 'checkbox';
  } else {
    $html_name = $name;
    $html_type = 'radio';
  }
  
  $output = '<table class="' . $css_prefix . '_chb_table">';
  $row_num = 0;
  $col_num = 0;
  $tr_num = 0;
  $id_num = 0;
  $markers = array();
  foreach ($values as $row) {
    if ($col_num == 0) {
      $markers[$row_num] = $default_array;
      $tr_num++;
      $markers[$row_num]['TBLROW_CLASS'] = $css_prefix . '_chb_tr_' . $tr_num;
    }
    
    $col_num++;
    
    $html_selected = '';
    if ($type == 'checkboxes') {
      if (is_array($selected)) {
        $html_selected = in_array($row[$value_key], $selected) ? ' checked="checked"' : '';
      }
    } else {
      $html_selected = $row[$value_key] == $selected ? ' checked="checked"' : '';
    }
    
    $markers[$row_num]['TBLCOL' . $col_num . '_CLASS'] = $css_prefix . '_chb_td_' . $col_num;
    $markers[$row_num]['TBLCOL' . $col_num . '_CONTENT'] = '<input type="' . $html_type . '" name="' . $html_name . '" value="' . $row[$value_key] . '" id="id_' . $name . '_' . $id_num . '"' . $html_selected . ' class="' . $css_prefix . '_input_' . $tr_num . '_' . $col_num . '" /><label for="id_' . $name . '_' . $id_num . '" class="' . $css_prefix . '_label_' . $tr_num . '_' . $col_num . '">' . $row[$caption_key] . '</label>';
    
    if ($col_num == $num_cols) {
      $col_num = 0;
      $row_num++;
    }
    
    $id_num++;
  }
  
  foreach($markers as $row_data) {
    $new_row = $tbl_row;
    foreach($row_data as $marker => $value) {
      $new_row = str_replace($marker, $value, $new_row);
    }
    $output .= $new_row;
  }
  
  $output .= '</table>';
  
  return $output;
}

?>