<?php
// Author: Jan Boldis

/* function=html_dateselect
  parameters:
    from = minimum year number
    to = maximum year number
    name = the name prefix of select-type html elements
    selected = array with <name>_year, <name>_month and <name>_day keys
    css_prefix = the prefix for html elements classes
    label_year = text for label for year selector
    label_month = text for label for month selector
    label_day = text for label for day selector
    use_month_names = can be boolean (true/false) or array (keys are numbers of months, values are names of months), if set this is set to true, the english names of months are used
*/
function swift_function_html_dateselect(&$swift, $params = array()) {
  $default_month_name = array( 1 => 'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');
  $from = (isset($params['from']) ? intval($params['from']) : date('Y') - 20);
  $to = (isset($params['to']) ? intval($params['to']) : date('Y') + 20);
  $selected = (isset($params['selected']) ? $params['selected'] : array());
  $name = (isset($params['name']) ? $params['name'] : 'swift_date');
  $css_prefix = (isset($params['css_prefix']) ? $params['css_prefix'] : 'swift_date');
  $label_year = (isset($params['label_year']) ? $params['label_year'] : 'Year');
  $label_month = (isset($params['label_month']) ? $params['label_month'] : 'Month');
  $label_day = (isset($params['label_day']) ? $params['label_day'] : 'Day');
  $use_month_names = (isset($params['use_month_names']) ? ($params['use_month_names'] === true ? $default_month_name : $params['use_month_names']) : false);
  
  $output = '<span class="' . $css_prefix . '_year"><select name="' . $name . '_year" size="1" id="id_' . $name . '_year">';
  for ($y=$to;$y>=$from;$y--) {
    $output .= '<option value="' . $y . '"' . (isset($selected[$name . '_year']) && $selected[$name . '_year'] == $y ? ' selected="selected"' : '') . '>' . $y . '</option>';
  }
  $output .= '</select><label for="id_' . $name . '_year">' . $label_year . '</label></span>';
  $output .= '<span class="' . $css_prefix . '_month"><select name="' . $name . '_month" size="1" id="id_' . $name . '_month">';
  for ($m=1;$m<=12;$m++) {
    $output .= '<option value="' . $m . '"' . (isset($selected[$name . '_month']) && $selected[$name . '_month'] == $m ? ' selected="selected"' : '') . '>' . ($use_month_names === false ? $m : (isset($use_month_names[$m]) ? $use_month_names[$m] : $default_month_name[$m])) . '</option>';
  }
  $output .= '</select><label for="id_' . $name . '_month">' . $label_month . '</label></span>';
  $output .= '<span class="' . $css_prefix . '_day"><select name="' . $name . '_day" size="1" id="id_' . $name . '_day">';
  for ($d=1;$d<=31;$d++) {
    $output .= '<option value="' . $d . '"' . (isset($selected[$name . '_day']) && $selected[$name . '_day'] == $d ? ' selected="selected"' : '') . '>' . $d . '</option>';
  }
  $output .= '</select><label for="id_' . $name . '_day">' . $label_day . '</label></span>';
  
  return $output;
}

?>