<?php

require_once('db_abstract.php');
require_once('collection_abstract.php');

class framework {

  private $_segments; // URL segmenty
  private $_urlData; // URL data
  private $_swift; // Objekt swift template engine
  private $_dblink; // Linka na pripojenie k DB
  private $_config; // konfiguracia zo suboru /etc/config.php
  private $_rewrite_module; // pouzitie rewrite modulu na skraslenie URL
  private $_queries_count = 0; // Pocet dotazov na databazu

  /*
   * Konstruktor
   * navrat: nic      
   */
  public function __construct() {
    $this->loadConfig();
    $this->_rewrite_module = $this->getConfig('rewrite_engine');
    $this->decodeUrl();
    $this->getUrlParams();
    $this->prepareTemplateEngine();
    $this->loadModel('sessions');
    $this->loadModel('resources');
    $this->connectDB();
  }
  
  /*
   * Destruktor
   * navrat: nic   
   */   
  public function __destruct() {
    if ($this->_dblink) {
      mysql_close($this->_dblink);
    }
  }
  
  /*
   * Start systemu
   * navrat: nic   
   */   
  public function start() {
    $controller = $this->getUrlSegment(0);
    $action = $this->getUrlSegment(1);
    $controller = $controller == '' ? 'index' : $controller;
    $action = $action == '' ? 'index' : $action;
    
    $this->callControllerAction($controller, $action);
    
    $this->sessions->clearFlashMessages();
  }
  
  /*
   * Ziskanie segmentu URL
   * $index (integer) - index segmentu, cislovane od nuly, ktory sa ma vyhladat a vratit
   * navrat: obsah segmentu URL s indexom $index    
   */   
  public function getUrlSegment($index = 0) {
    if (isset($this->_segments[intval($index)])) {
      return $this->_segments[intval($index)]; 
    }
    return '';
  }
  
  /*
   * Vrati vsetky dvojice segmentov ako premenna=hodnota zacinajuce od vstupneho indexu segmentu
   * $from (integer) - index segmentu, najlepsie 2, odkial sa zacne parovat
   * navrat: asociativne pole obsahujuce dvojice segmentu od pociatku vratane v tvare [premenna] => hodnota z /premenna/hodnota/     
   */   
  public function getUrlParams($from = 2) {
    if (isset($this->_urlData[intval($from)])) {
      $output = $this->_urlData[intval($from)];
    } else {
      $data = array();
      
      for ($i=intval($from);$i<count($this->_segments);$i++) {
        $data[] = $this->_segments[$i];
      }
      
      $output = array();
      
      for ($i=0;$i<count($data);$i+=2) {
        if (isset($data[$i])) { $output[$data[$i]] = isset($data[$i+1]) ? $data[$i+1] : ''; }
      }
      
      $this->_urlData[intval($from)] = $output;
    }
    
    return $output;
  }
  
  /*
   * Vrati url link v tvare /index.php/$controller/$action/$params
   * $controller (string) - nazov controllera, na ktory sa ma link smerovat
   * $action (string) - nazov akcie pre $controller, ktora sa ma vykonat
   * $params (array) - pole dvojic [premenna] => hodnota, ktore sa zakoduju do URL linku ako segmenty URL
   * navrat: link na dany $controller/$action s danymi parametrami $params           
   */   
  public function makeLinkUrl($controller = 'index', $action = 'index', $params = array()) {
    if ($this->_rewrite_module === true) {
      $link = '/';
    } else {
      $link = '/index.php/';
    }
  
    $link .= rawurlencode($controller) . '/' . rawurlencode($action) . '/';
    
    if (count($params)) {
      foreach($params as $var => $value) {
        if (is_numeric($value) || is_string($value)) {
          $link .= rawurlencode($var) . '/' . rawurlencode($value) . '/';
        }
      }
    }
    
    return $link;
  }
  
  /*
   * Vrati html anchor (<a>) element odkazujuci na dany $controller/$action/$params
   * $caption (string) - text medzi znackamy elementu anchor   
   * $controller (string) - nazov controllera, na ktory sa ma link smerovat
   * $action (string) - nazov akcie pre $controller, ktora sa ma vykonat
   * $params (array) - pole dvojic [premenna] => hodnota, ktore sa zakoduju do URL linku ako segmenty URL
   * $html_params (array) - html parametre elementu anchor, napr.: [target] => _blank
   * navrat: kompletny html element          
   */   
  public function makeLinkElement($caption, $controller = 'index', $action = 'index', $params = array(), $html_params = array()) {
    $link = $this->makeLinkUrl($controller, $action, $params);
    
    $output = '<a href="' . $link . '"';
    
    if (count($html_params)) {
      foreach($html_params as $param => $value) {
        $output .= ' ' . $param . '="' . addslashes($value) . '"';
      }
    }
    $output .= '>' . $caption . '</a>';
    
    return $output;
  }
  
  /*
   * Spusti controller a jeho akciu
   * $controller (string) - nazov controlleru, ktory sa ma spustit
   * $action (string) - nazov akcie controlleru, ktora sa ma vykonat
   * navrat: nic         
   */   
  public function callControllerAction($controller = 'index', $action = 'index') {
    $controller_controller = $controller . '_controller';
    $action_action = $action . '_action';
    
    if (file_exists('controllers/' . $controller . '.php')) {
      include_once('controllers/' . $controller . '.php');
      
      if (class_exists($controller_controller)) {
        $c = new $controller_controller($this);
        
        if (method_exists($c, $action_action)) {
          $c->$action_action();
        } else {
          echo 'Invalid action <strong>' . $action . '</strong>!';
        }
      } else {
        echo 'There is no <strong>' . $controller_controller . '</strong> class defined!';
      }
    } else {
      echo 'Controller <strong>' . $controller . '</strong> not found!';
    }
  }
  
  /*
   * Http presmerovanie na dany $controller/$action/$params
   * $controller (string) - nazov controlleru, na ktory sa ma presmerovat
   * $action (string) - akcia controlleru, ktora sa ma vykonat
   * $params (array) - pole parametrov, ktore sa maju pridat k URL
   * navrat: nic            
   */   
  public function redirectTo($controller = 'index', $action = 'index', $params = array()) {
    $link = $this->makeLinkUrl($controller, $action, $params);
    
    header('location: ' . $link);
    die();
  }
  
  /*
   * Priradi premennu s obsahom do swift template engine objektu
   * $variable (string) - nazov premennej
   * $value (mixed) - hodnota premennej
   * navrat: nic         
   */   
  public function templateAssign($variable, $value) {
    $this->_swift->assign($variable, $value);
  }
  
  /*
   * Priradi pole premennych do swift template engine objektu
   * $array (array) - pole premennych
   * navrat: nic      
   */   
  public function templateAssignArray($array) {
    $this->_swift->assign_array($array);
  }
  
  /*
   * Vymaze premennu predtym priradenu do swift template engine objektu
   * $variable (string) - nazov premennej, ktora sa ma vymazat
   * navrat: nic      
   */   
  public function templateUnassign($variable) {
    $this->_swift->unassign($variable);
  }
  
  /*
   * Posle na vystup spracovanu sablonu
   * $file (string) - [cesta a] nazov k suboru sablony, ktora sa ma spracovat a zobrazit
   * navrat: nic     
   */   
  public function templateDisplay($file) {
    $this->templateAssign('system_queries_count', $this->_queries_count);
    echo $this->_swift->parse($file);
  }
  
  /* 
   * Vrati vystup po spracovani sablony
   * $file (string) - [cesta a] nazov k suboru sablony, ktora sa ma spracovat a vratit
   * navrat: spracovany obsah sablony      
   */   
  public function templateParse($file) {
    $this->templateAssign('system_queries_count', $this->_queries_count);
    return $this->_swift->parse($file);
  }
  
  /*
   * Vymaze vsetky skompilovane sablony
   * navrat: nic   
   */   
  public function templateClearAll() {
    $this->_swift->clear_all_compiled();
  }
  
  /*
   * Zapne debug mod
   * navrat: nic   
   */   
  public function templateDebug() {
    $this->_swift->setDebuging(true);
  }
  
  /*
   * Nacita a inicializuje objekt modelu
   * $name (string) - nazov modelu, ktory sa ma nacitat
   * navrat: nic;     
   */   
  public function loadModel($name) {
    if (isset($this->$name)) { return; }
    
    if (file_exists('models/' . $name . '.php')) {
      include_once 'models/' . $name . '.php';
      
      if (class_exists($name)) {
        $this->$name = new $name($this);
      } else {
        echo 'There is no <strong>' . $name . '</strong> class defined!';
      }
    } else {
      echo 'Model <strong>' . $name . '</strong> not found!';
    }
  }
  
  /*
   * Vytvori novu instanciu riadka tabulky
   * $name (string) - nazov tabulky v db / nazov triedy pre pracu s riadkom tabulky
   * $data (array) - vychodzie data   
   * navrat: referencia na novy objekt riadka tabulky      
   */   
  public function instanceOfDbTable($name, $data = array()) {
    $tablename = strtolower($name) . '_db';
    
    if (file_exists('database/' . $tablename . '.php')) {
      include_once('database/' . $tablename . '.php');
      
      if (class_exists($tablename)) {
        return new $tablename($this, $this->_dblink, $data);
      } else {
        echo 'There is no class definition <strong>' . $tablename . '</strong> for database table!';
        die();
      }
    } else {
      echo 'There is no <strong>' . $name . '</strong> database table definition!';
      die();
    }
  }
  
  /*
   * Vytvori novu instanciu kolekcie tabulky
   * $name (string) - meno tabulky v databazi
   * navrat: referencia na novy objekt kolekcie tabulky      
   */   
  public function instanceOfTableCollection($name) {
    $tablename = strtolower($name) . '_collection';
    
    if (file_exists('database/' . $tablename . '.php')) {
      include_once('database/' . $tablename . '.php');
      
      if (class_exists($tablename)) {
        return new $tablename($this, $this->_dblink);
      } else {
        echo 'There is no class definition <strong>' . $tablename . '</strong> for database collection!';
        die();
      }
    } else {
      echo 'There is no <strong>' . $name . '</strong> database collection definition!';
      die();
    }
  }
  
  /*
   * Vrati konkretnu hodnotu konfiguracneho pola alebo podpole konfiguracneho pola
   * * (strings) - dynamicke retazcove parametre s nazvami klucov v konfiguracnom poli
   * navrat: string alebo array      
   */   
  public function getConfig() {
    if (func_num_args() == 0) {
      return $this->_config;
    } else {
      $args = func_get_args();
      return $this->recursiveGetConfig($this->_config, $args);
    }
  }
  
  /*
   * Zvisi pocitadlo dotazov o jedna
   * navrat: nic   
   */   
  public function incQueries() {
    $this->_queries_count++;
  }
  
  /*
   * Rekurzivne prehlada pole konfiguracie a vrati pozadovanu hodnotu alebo podpole
   * $config_array (array) - konfiguracne pole alebo podpole konfiguracneho pola
   * $arguments_array (array) - zostavajuce argumenty (kluce v konfiguracnom poli)
   * navrat: string alebo array podla toho zadanych argumentov        
   */   
  private function recursiveGetConfig($config_array, $arguments_array) {
    if (!count($arguments_array)) {
      return $config_array;
    } else {
      reset($arguments_array);
      $key = key($arguments_array);
      $current_arg = $arguments_array[$key];
      unset($arguments_array[$key]);
      if (isset($config_array[$current_arg])) {
    	  return $this->recursiveGetConfig($config_array[$current_arg], $arguments_array);
    	} else {
        return '';
      }
    }
  }
  
  /*
   * Nacita konfiguraciu z /etc/config.php
   * navart: nic   
   */   
  private function loadConfig() {
    require_once('etc/config.php');
    $this->_config = $config;
  }
  
  /*
   * Dekoduje a ulozi do pamati URL segmenty
   * navrat: nic   
   */   
  private function decodeUrl() {
    $path_info = $_SERVER['PATH_INFO'];
    if (strpos($path_info, '?') !== false) {
      list($path_info, $unusable_data) = explode('?', $path_info, 2);
    }
    $path_info = $path_info[0] == '/' ? substr($path_info, 1) : $path_info;
    $path_info = substr($path_info, -1) == '/' ? substr($path_info, 0, -1) : $path_info;
    
    $this->_segments = explode('/', $path_info);
  }
  
  /*
   * Inicializuje swift template engine
   * navrat: nic   
   */   
  private function prepareTemplateEngine() {
    require_once 'system/swift/swift/class.swift.php';
    
    $this->_swift = new swift();
    $this->_swift->setTPLdir('./views');
    $this->_swift->setCMPdir('./system/swift/compiled');
    $this->_swift->setSWFdir('./system/swift/swift');
    
    if ($this->getConfig('template', 'clear_all_cache') === true) {
      $this->templateClearAll();
    }
  }
  
  /*
   * Inicializuje pripojenie na DB
   * navrat: nic   
   */   
  private function connectDB() {
    //if ($this->_dblink = mysql_connect($config['db']['host'], $config['db']['user'], $config['db']['pw'])) {
    if ($this->_dblink = mysql_connect($this->getConfig('db', 'host'), $this->getConfig('db', 'user'), $this->getConfig('db', 'pw'))) {
      //if (!mysql_select_db($config['db']['dbname'], $this->_dblink)) {
      if (!mysql_select_db($this->getConfig('db', 'dbname'), $this->_dblink)) {
        echo '<strong>Warning:</strong> Can\'t select database.<br />';
      }
    } else {
      echo '<strong>Warning:</strong> Database conection failed.<br />';
    }
  }
}

?>